# encoding=utf8

"""Implementation of exceptions."""


class FesException(Exception):
    r"""Exception for exceeding number of maximum function evaluations.

    Author:
            Klemen Berkovič

    Date:
            2018

    License:
            MIT

    See Also:
            * :class:`Exception`

    """

    def __init__(self, message='Reached the allowed number of the function evaluations!!!'):
        r"""Initialize the exception.

        Args:
                message (Optional[str]): Message show when this exception is thrown

        """
        super().__init__(message)


class GenException(Exception):
    r"""Exception for exceeding number of algorithm iterations/generations.

    Author:
            Klemen Berkovič

    Date:
            2018

    License:
            MIT

    See Also:
            * :class:`Exception`

    """

    def __init__(self, message='Reached the allowed number of the algorithm evaluations!!!'):
        r"""Initialize the exception.

        Args:
                message (Optional[str]): Message that is shown when this exceptions is thrown

        """
        super().__init__(message)


class TimeException(Exception):
    r"""Exception for exceeding time limit.

    Author:
            Klemen Berkovič

    Date:
            2018

    License:
            MIT

    See Also:
            * :class:`Exception`

    """

    def __init__(self, message='Reached the allowed run time of the algorithm'):
        r"""Initialize the exception.

        Args:
                message (Optional[str]): Message that is show when this exception is thrown.

        """
        super().__init__(message)


class RefException(Exception):
    r"""Exception for exceeding reference value of function/fitness value.

    Author:
            Klemen Berkovič

    Date:
            2018

    License:
            MIT

    See Also:
            * :class:`Exception`

    """

    def __init__(self, message='Reached the reference point!!!'):
        r"""Initialize the exception.

        Args:
                message (Optional[str]): Message that is show when this exception is thrown.

        """
        super().__init__(message)



# encoding=utf8

"""Implementations of Schaffer functions."""

from math import sin, cos, sqrt
from niapy.benchmarks.benchmark import Benchmark

__all__ = ['SchafferN2', 'SchafferN4', 'ExpandedSchaffer']


class SchafferN2(Benchmark):
    r"""Implementations of Schaffer N. 2 functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Schaffer N. 2 Function**
    :math:`f(\textbf{x}) = 0.5 + \frac{ \sin^2 \left( x_1^2 - x_2^2 \right) - 0.5 }{ \left( 1 + 0.001 \left(  x_1^2 + x_2^2 \right) \right)^2 }`

    **Input domain:**
    The function can be defined on any input domain but it is usually
    evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.

    **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (420.968746,...,420.968746)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = 0.5 + \frac{ \sin^2 \left( x_1^2 - x_2^2 \right) - 0.5 }{ \left( 1 + 0.001 \left(  x_1^2 + x_2^2 \right) \right)^2 }$

        Equation:
            \begin{equation} f(\textbf{x}) = 0.5 + \frac{ \sin^2 \left( x_1^2 - x_2^2 \right) - 0.5 }{ \left( 1 + 0.001 \left(  x_1^2 + x_2^2 \right) \right)^2 } \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference:
        http://www5.zzu.edu.cn/__local/A/69/BC/D3B5DFE94CD2574B38AD7CD1D12_C802DAFE_BC0C0.pdf

    """

    Name = ['SchafferN2']

    def __init__(self, lower=-100.0, upper=100.0):
        r"""Initialize of SchafferN2 benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = 0.5 + \frac{ \sin^2 \left( x_1^2 - x_2^2 \right) - 0.5 }{ \left( 1 + 0.001 \left(  x_1^2 + x_2^2 \right) \right)^2 }$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def f(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            return 0.5 + (sin(x[0] ** 2 - x[1] ** 2) ** 2 - 0.5) / (1 + 0.001 * (x[0] ** 2 + x[1] ** 2)) ** 2

        return f


class SchafferN4(Benchmark):
    r"""Implementations of Schaffer N. 2 functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Schaffer N. 2 Function**
    :math:`f(\textbf{x}) = 0.5 + \frac{ \cos^2 \left( \sin \left( x_1^2 - x_2^2 \right) \right)- 0.5 }{ \left( 1 + 0.001 \left(  x_1^2 + x_2^2 \right) \right)^2 }`

    **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.

    **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (420.968746,...,420.968746)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = 0.5 + \frac{ \cos^2 \left( \sin \left( x_1^2 - x_2^2 \right) \right)- 0.5 }{ \left( 1 + 0.001 \left(  x_1^2 + x_2^2 \right) \right)^2 }$

        Equation:
            \begin{equation} f(\textbf{x}) = 0.5 + \frac{ \cos^2 \left( \sin \left( x_1^2 - x_2^2 \right) \right)- 0.5 }{ \left( 1 + 0.001 \left(  x_1^2 + x_2^2 \right) \right)^2 } \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference:
        http://www5.zzu.edu.cn/__local/A/69/BC/D3B5DFE94CD2574B38AD7CD1D12_C802DAFE_BC0C0.pdf

    """

    Name = ['SchafferN4']

    def __init__(self, lower=-100.0, upper=100.0):
        r"""Initialize of SchafferN4 benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = 0.5 + \frac{ \cos^2 \left( \sin \left( x_1^2 - x_2^2 \right) \right)- 0.5 }{ \left( 1 + 0.001 \left(  x_1^2 + x_2^2 \right) \right)^2 }$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def f(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            return 0.5 + (cos(sin(x[0] ** 2 - x[1] ** 2)) ** 2 - 0.5) / (1 + 0.001 * (x[0] ** 2 + x[1] ** 2)) ** 2

        return f


class ExpandedSchaffer(Benchmark):
    r"""Implementations of Expanded Schaffer functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
        **Expanded Schaffer Function**
        :math:`f(\textbf{x}) = g(x_D, x_1) + \sum_{i=2}^D g(x_{i - 1}, x_i) \\ g(x, y) = 0.5 + \frac{\sin \left(\sqrt{x^2 + y^2} \right)^2 - 0.5}{\left( 1 + 0.001 (x^2 + y^2) \right)}^2`

    **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.

    **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (420.968746,...,420.968746)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = g(x_D, x_1) + \sum_{i=2}^D g(x_{i - 1}, x_i) \\ g(x, y) = 0.5 + \frac{\sin \left(\sqrt{x^2 + y^2} \right)^2 - 0.5}{\left( 1 + 0.001 (x^2 + y^2) \right)}^2$

        Equation:
            \begin{equation} f(\textbf{x}) = g(x_D, x_1) + \sum_{i=2}^D g(x_{i - 1}, x_i) \\ g(x, y) = 0.5 + \frac{\sin \left(\sqrt{x^2 + y^2} \right)^2 - 0.5}{\left( 1 + 0.001 (x^2 + y^2) \right)}^2 \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference:
        http://www5.zzu.edu.cn/__local/A/69/BC/D3B5DFE94CD2574B38AD7CD1D12_C802DAFE_BC0C0.pdf

    """

    Name = ['ExpandedSchaffer']

    def __init__(self, lower=-100.0, upper=100.0):
        r"""Initialize of Expanded Schaffer benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = g(x_D, x_1) + \sum_{i=2}^D g(x_{i - 1}, x_i) \\ g(x, y) = 0.5 + \frac{\sin \left(\sqrt{x^2 + y^2} \right)^2 - 0.5}{\left( 1 + 0.001 (x^2 + y^2) \right)}^2$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def g(x, y):
            return 0.5 + (sin(sqrt(x ** 2 + y ** 2)) ** 2 - 0.5) / (1 + 0.001 * (x ** 2 + y ** 2)) ** 2

        def f(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            val = 0.0
            for i in range(1, dimension):
                val += g(x[i - 1], x[i])
            return g(x[dimension - 1], x[0]) + val

        return f

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8

"""Implementations of Schwefel's functions."""

from math import sin, fmod, fabs, sqrt
from niapy.benchmarks.benchmark import Benchmark

__all__ = ['Schwefel', 'Schwefel221', 'Schwefel222', 'ModifiedSchwefel']


class Schwefel(Benchmark):
    r"""Implementation of Schwefel function.

    Date: 2018

    Author: Lucija Brezočnik

    License: MIT

    Function: **Schwefel function**

        :math:`f(\textbf{x}) = 418.9829d - \sum_{i=1}^{D} x_i \sin(\sqrt{\lvert x_i \rvert})`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-500, 500]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (420.968746,...,420.968746)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = 418.9829d - \sum_{i=1}^{D} x_i \sin(\sqrt{\lvert x_i \rvert})$

        Equation:
            \begin{equation} f(\textbf{x}) = 418.9829d - \sum_{i=1}^{D} x_i
            \sin(\sqrt{\lvert x_i \rvert}) \end{equation}

        Domain:
            $-500 \leq x_i \leq 500$

    Reference:
        https://www.sfu.ca/~ssurjano/schwef.html

    """

    Name = ['Schwefel']

    def __init__(self, lower=-500.0, upper=500.0):
        r"""Initialize of Schwefel benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = 418.9829d - \sum_{i=1}^{D} x_i \sin(\sqrt{\lvert x_i \rvert})$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def evaluate(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            val = 0.0
            for i in range(dimension):
                val += (x[i] * sin(sqrt(abs(x[i]))))
            return 418.9829 * dimension - val

        return evaluate


class Schwefel221(Benchmark):
    r"""Schwefel 2.21 function implementation.

    Date: 2018

    Author: Grega Vrbančič

    Licence: MIT

    Function: **Schwefel 2.21 function**

        :math:`f(\mathbf{x})=\max_{i=1,...,D}|x_i|`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (0,...,0)`

    LaTeX formats:
        Inline:
            $f(\mathbf{x})=\max_{i=1,...,D} \lvert x_i \rvert$

        Equation:
            \begin{equation}f(\mathbf{x}) = \max_{i=1,...,D} \lvert x_i \rvert \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference paper:
        Jamil, M., and Yang, X. S. (2013).
        A literature survey of benchmark functions for global optimisation problems.
        International Journal of Mathematical Modelling and Numerical Optimisation,
        4(2), 150-194.

    """

    Name = ['Schwefel221']

    def __init__(self, lower=-100.0, upper=100.0):
        r"""Initialize of Schwefel221 benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\mathbf{x})=\max_{i=1,...,D} \lvert x_i \rvert$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def evaluate(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                Callable[[int, numpy.ndarray, dict], float]: Fitness function.

            """
            maximum = 0.0
            for i in range(dimension):
                if abs(x[i]) > maximum:
                    maximum = abs(x[i])
            return maximum

        return evaluate


class Schwefel222(Benchmark):
    r"""Schwefel 2.22 function implementation.

    Date: 2018

    Author: Grega Vrbančič

    Licence: MIT

    Function: **Schwefel 2.22 function**

        :math:`f(\mathbf{x})=\sum_{i=1}^{D} \lvert x_i \rvert +\prod_{i=1}^{D} \lvert x_i \rvert`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (0,...,0)`

    LaTeX formats:
        Inline:
            $f(\mathbf{x})=\sum_{i=1}^{D} \lvert x_i \rvert +\prod_{i=1}^{D} \lvert x_i \rvert$

        Equation:
            \begin{equation}f(\mathbf{x}) = \sum_{i=1}^{D} \lvert x_i \rvert + \prod_{i=1}^{D} \lvert x_i \rvert \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference paper:
        Jamil, M., and Yang, X. S. (2013).
        A literature survey of benchmark functions for global optimisation problems.
        International Journal of Mathematical Modelling and Numerical Optimisation,
        4(2), 150-194.

    """

    Name = ['Schwefel222']

    def __init__(self, lower=-100.0, upper=100.0):
        r"""Initialize of Schwefel222 benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\mathbf{x})=\sum_{i=1}^{D} \lvert x_i \rvert +\prod_{i=1}^{D} \lvert x_i \rvert$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def evaluate(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            part1 = 0.0
            part2 = 1.0
            for i in range(dimension):
                part1 += abs(x[i])
                part2 *= abs(x[i])
            return part1 + part2

        return evaluate


class ModifiedSchwefel(Benchmark):
    r"""Implementations of Modified Schwefel functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Modified Schwefel Function**

        :math:`f(\textbf{x}) = 418.9829 \cdot D - \sum_{i=1}^D h(x_i) \\ h(x) = g(x + 420.9687462275036)  \\ g(z) = \begin{cases} z \sin \left( \lvert z \rvert^{\frac{1}{2}} \right) &\quad \lvert z \rvert \leq 500 \\ \left( 500 - \mod (z, 500) \right) \sin \left( \sqrt{\lvert 500 - \mod (z, 500) \rvert} \right) - \frac{ \left( z - 500 \right)^2 }{ 10000 D }  &\quad z > 500 \\ \left( \mod (\lvert z \rvert, 500) - 500 \right) \sin \left( \sqrt{\lvert \mod (\lvert z \rvert, 500) - 500 \rvert} \right) + \frac{ \left( z - 500 \right)^2 }{ 10000 D } &\quad z < -500\end{cases}`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (420.968746,...,420.968746)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = 418.9829 \cdot D - \sum_{i=1}^D h(x_i) \\ h(x) = g(x + 420.9687462275036)  \\ g(z) = \begin{cases} z \sin \left( \lvert z \rvert^{\frac{1}{2}} \right) &\quad \lvert z \rvert \leq 500 \\ \left( 500 - \mod (z, 500) \right) \sin \left( \sqrt{\lvert 500 - \mod (z, 500) \rvert} \right) - \frac{ \left( z - 500 \right)^2 }{ 10000 D }  &\quad z > 500 \\ \left( \mod (\lvert z \rvert, 500) - 500 \right) \sin \left( \sqrt{\lvert \mod (\lvert z \rvert, 500) - 500 \rvert} \right) + \frac{ \left( z - 500 \right)^2 }{ 10000 D } &\quad z < -500\end{cases}$

        Equation:
            \begin{equation} f(\textbf{x}) = 418.9829 \cdot D - \sum_{i=1}^D h(x_i) \\ h(x) = g(x + 420.9687462275036)  \\ g(z) = \begin{cases} z \sin \left( \lvert z \rvert^{\frac{1}{2}} \right) &\quad \lvert z \rvert \leq 500 \\ \left( 500 - \mod (z, 500) \right) \sin \left( \sqrt{\lvert 500 - \mod (z, 500) \rvert} \right) - \frac{ \left( z - 500 \right)^2 }{ 10000 D }  &\quad z > 500 \\ \left( \mod (\lvert z \rvert, 500) - 500 \right) \sin \left( \sqrt{\lvert \mod (\lvert z \rvert, 500) - 500 \rvert} \right) + \frac{ \left( z - 500 \right)^2 }{ 10000 D } &\quad z < -500\end{cases} \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference:
        http://www5.zzu.edu.cn/__local/A/69/BC/D3B5DFE94CD2574B38AD7CD1D12_C802DAFE_BC0C0.pdf

    """

    Name = ['ModifiedSchwefel']

    def __init__(self, lower=-100.0, upper=100.0):
        r"""Initialize of Modified Schwefel benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = 418.9829 \cdot D - \sum_{i=1}^D h(x_i) \\ h(x) = g(x + 420.9687462275036)  \\ g(z) = \begin{cases} z \sin \left( \lvert z \rvert^{\frac{1}{2}} \right) &\quad \lvert z \rvert \leq 500 \\ \left( 500 - \mod (z, 500) \right) \sin \left( \sqrt{\lvert 500 - \mod (z, 500) \rvert} \right) - \frac{ \left( z - 500 \right)^2 }{ 10000 D }  &\quad z > 500 \\ \left( \mod (\lvert z \rvert, 500) - 500 \right) \sin \left( \sqrt{\lvert \mod (\lvert z \rvert, 500) - 500 \rvert} \right) + \frac{ \left( z - 500 \right)^2 }{ 10000 D } &\quad z < -500\end{cases}$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def g(z, dimension):
            if z > 500:
                return (500 - fmod(z, 500)) * sin(sqrt(fabs(500 - fmod(z, 500)))) - (z - 500) ** 2 / (10000 * dimension)
            elif z < -500:
                return (fmod(z, 500) - 500) * sin(sqrt(fabs(fmod(z, 500) - 500))) + (z - 500) ** 2 / (10000 * dimension)
            return z * sin(fabs(z) ** (1 / 2))

        def h(x, dimension):
            return g(x + 420.9687462275036, dimension)

        def f(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem.
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            val = 0.0
            for i in range(dimension):
                val += h(x[i], dimension)
            return 418.9829 * dimension - val

        return f

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8

"""Implementations of Bent Cigar functions."""

from niapy.benchmarks.benchmark import Benchmark

__all__ = ['BentCigar']


class BentCigar(Benchmark):
    r"""Implementations of Bent Cigar functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Bent Cigar Function**

        :math:`f(\textbf{x}) = x_1^2 + 10^6 \sum_{i=2}^D x_i^2`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        :math:`f(x^*) = 0`, at :math:`x^* = (420.968746,...,420.968746)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = x_1^2 + 10^6 \sum_{i=2}^D x_i^2$

        Equation:
            \begin{equation} f(\textbf{x}) = x_1^2 + 10^6 \sum_{i=2}^D x_i^2 \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference:
        http://www5.zzu.edu.cn/__local/A/69/BC/D3B5DFE94CD2574B38AD7CD1D12_C802DAFE_BC0C0.pdf

    """

    Name = ['BentCigar']

    def __init__(self, lower=-100.0, upper=100.0):
        r"""Initialize of Bent Cigar benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = x_1^2 + 10^6 \sum_{i=2}^D x_i^2$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function

        """
        def f(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            val = 0.0
            for i in range(1, dimension):
                val += x[i] ** 2
            return x[0] ** 2 + 10 ** 6 * val

        return f

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8

"""Implementation of SchumerSteiglitz function."""

import math
from niapy.benchmarks.benchmark import Benchmark

__all__ = ['SchumerSteiglitz']


class SchumerSteiglitz(Benchmark):
    r"""Implementation of Schumer Steiglitz function.

    Date: 2018

    Author: Lucija Brezočnik

    License: MIT

    Function: **Schumer Steiglitz function**

        :math:`f(\mathbf{x}) = \sum_{i=1}^D x_i^4`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum** :math:`f(x^*) = 0`, at :math:`x^* = (0,...,0)`

    LaTeX formats:
        Inline:
            $f(\mathbf{x}) = \sum_{i=1}^D x_i^4$

        Equation:
            \begin{equation} f(\mathbf{x}) = \sum_{i=1}^D x_i^4 \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference paper:
        Jamil, M., and Yang, X. S. (2013).
        A literature survey of benchmark functions for global optimisation problems.
        International Journal of Mathematical Modelling and Numerical Optimisation,
        4(2), 150-194.

    """

    Name = ['SchumerSteiglitz']

    def __init__(self, lower=-100.0, upper=100.0):
        r"""Initialize of Schumer Steiglitz benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\mathbf{x}) = \sum_{i=1}^D x_i^4$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def evaluate(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            val = 0.0

            for i in range(dimension):
                val += math.pow(x[i], 4)

            return val

        return evaluate



# encoding=utf8
"""Implementations of Zakharov function."""

from niapy.benchmarks.benchmark import Benchmark

__all__ = ['Zakharov']


class Zakharov(Benchmark):
    r"""Implementations of Zakharov functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Zakharov Function**

        :math:`f(\textbf{x}) = \sum_{i = 1}^D x_i^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^4`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-5, 10]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        :math:`f(\textbf{x}^*) = 0` at :math:`\textbf{x}^* = (0, \cdots, 0)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = \sum_{i = 1}^D x_i^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^4$

        Equation:
            \begin{equation} f(\textbf{x}) = \sum_{i = 1}^D x_i^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^4 \end{equation}

        Domain:
            $-5 \leq x_i \leq 10$

    Reference:
        https://www.sfu.ca/~ssurjano/zakharov.html

    """

    Name = ['Zakharov']

    def __init__(self, lower=-5.0, upper=10.0):
        r"""Initialize of Zakharov benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = \sum_{i = 1}^D x_i^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^2 + \left( \sum_{i = 1}^D 0.5 i x_i \right)^4$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def f(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            v1, v2 = 0.0, 0.0
            for i in range(dimension):
                v1, v2 = v1 + x[i] ** 2, v2 + 0.5 * (i + 1) * x[i]
            return v1 + v2 ** 2 + v2 ** 4

        return f

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8

"""Implementation of Pinter function."""

import math
from niapy.benchmarks.benchmark import Benchmark

__all__ = ['Pinter']


class Pinter(Benchmark):
    r"""Implementation of Pintér function.

    Date: 2018

    Author: Lucija Brezočnik

    License: MIT

    Function: **Pintér function**

        :math:`f(\mathbf{x}) =
        \sum_{i=1}^D ix_i^2 + \sum_{i=1}^D 20i \sin^2 A + \sum_{i=1}^D i
        \log_{10} (1 + iB^2);`
        :math:`A = (x_{i-1}\sin(x_i)+\sin(x_{i+1}))\quad \text{and} \quad`
        :math:`B = (x_{i-1}^2 - 2x_i + 3x_{i+1} - \cos(x_i) + 1)`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-10, 10]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (0,...,0)`

    LaTeX formats:
        Inline:
            $f(\mathbf{x}) =
            \sum_{i=1}^D ix_i^2 + \sum_{i=1}^D 20i \sin^2 A + \sum_{i=1}^D i
            \log_{10} (1 + iB^2);
            A = (x_{i-1}\sin(x_i)+\sin(x_{i+1}))\quad \text{and} \quad
            B = (x_{i-1}^2 - 2x_i + 3x_{i+1} - \cos(x_i) + 1)$

        Equation:
            \begin{equation} f(\mathbf{x}) = \sum_{i=1}^D ix_i^2 +
            \sum_{i=1}^D 20i \sin^2 A + \sum_{i=1}^D i \log_{10} (1 + iB^2);
            A = (x_{i-1}\sin(x_i)+\sin(x_{i+1}))\quad \text{and} \quad
            B = (x_{i-1}^2 - 2x_i + 3x_{i+1} - \cos(x_i) + 1) \end{equation}

        Domain:
            $-10 \leq x_i \leq 10$

    Reference paper:
        Jamil, M., and Yang, X. S. (2013).
        A literature survey of benchmark functions for global optimisation problems.
        International Journal of Mathematical Modelling and Numerical Optimisation,
        4(2), 150-194.

    """

    Name = ['Pinter']

    def __init__(self, lower=-10.0, upper=10.0):
        r"""Initialize of Pinter benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r''' $f(\mathbf{x}) =
                \sum_{i=1}^D ix_i^2 + \sum_{i=1}^D 20i \sin^2 A + \sum_{i=1}^D i
                \log_{10} (1 + iB^2);
                A = (x_{i-1}\sin(x_i)+\sin(x_{i+1}))\quad \text{and} \quad
                B = (x_{i-1}^2 - 2x_i + 3x_{i+1} - \cos(x_i) + 1)$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def evaluate(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            val1 = 0.0
            val2 = 0.0
            val3 = 0.0

            for i in range(dimension):

                if i == 0:
                    sub = x[dimension - 1]
                    add = x[i + 1]
                elif i == dimension - 1:
                    sub = x[i - 1]
                    add = x[0]
                else:
                    sub = x[i - 1]
                    add = x[i + 1]

                a = (sub * math.sin(x[i]) + math.sin(add))
                b = (math.pow(sub, 2) - 2.0 * x[i] + 3.0 * add - math.cos(x[i]) + 1.0)

                val1 += (i + 1.0) * math.pow(x[i], 2)
                val2 += 20.0 * (i + 1.0) * math.pow(math.sin(a), 2)
                val3 += (i + 1.0) * math.log10(1.0 + (i + 1.0) * math.pow(b, 2))

            return val1 + val2 + val3

        return evaluate



# encoding=utf8
from niapy.algorithms.basic import CuckooSearch
from niapy.tests.test_algorithm import AlgorithmTestCase, MyBenchmark


class CSTestCase(AlgorithmTestCase):
    def setUp(self):
        AlgorithmTestCase.setUp(self)
        self.algo = CuckooSearch

    def test_custom(self):
        cs_custom = self.algo(population_size=20, seed=self.seed)
        cs_customc = self.algo(population_size=20, seed=self.seed)
        AlgorithmTestCase.test_algorithm_run(self, cs_custom, cs_customc, MyBenchmark())

    def test_griewank(self):
        cs_griewank = self.algo(population_size=10, seed=self.seed)
        cs_griewankc = self.algo(population_size=10, seed=self.seed)
        AlgorithmTestCase.test_algorithm_run(self, cs_griewank, cs_griewankc)

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8

"""Implementations of Trid function."""

from niapy.benchmarks.benchmark import Benchmark

__all__ = ['Trid']


class Trid(Benchmark):
    r"""Implementations of Trid functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Trid Function**

        :math:`f(\textbf{x}) = \sum_{i = 1}^D \left( x_i - 1 \right)^2 - \sum_{i = 2}^D x_i x_{i - 1}`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-D^2, D^2]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        :math:`f(\textbf{x}^*) = \frac{-D(D + 4)(D - 1)}{6}` at :math:`\textbf{x}^* = (1 (D + 1 - 1), \cdots , i (D + 1 - i) , \cdots , D (D + 1 - D))`

    LaTeX formats:
        Inline:
                $f(\textbf{x}) = \sum_{i = 1}^D \left( x_i - 1 \right)^2 - \sum_{i = 2}^D x_i x_{i - 1}$

        Equation:
                \begin{equation} f(\textbf{x}) = \sum_{i = 1}^D \left( x_i - 1 \right)^2 - \sum_{i = 2}^D x_i x_{i - 1} \end{equation}

        Domain:
                $-D^2 \leq x_i \leq D^2$

    Reference:
        https://www.sfu.ca/~ssurjano/trid.html

    """

    Name = ['Trid']

    def __init__(self, dimension=2):
        r"""Initialize of Trid benchmark.

        Args:
            dimension (int): Dimension of the problem used to determine lower and upper bounds.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(-(dimension ** 2), dimension ** 2)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = \sum_{i = 1}^D \left( x_i - 1 \right)^2 - \sum_{i = 2}^D x_i x_{i - 1}$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def f(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            v1, v2 = 0.0, 0.0
            for i in range(dimension):
                v1 += (x[i] - 1) ** 2
            for i in range(1, dimension):
                v2 += x[i] * x[i - 1]
            return v1 - v2

        return f

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8

"""Implementations of HGBat functions."""

from math import fabs
from niapy.benchmarks.benchmark import Benchmark

__all__ = ['HGBat']


class HGBat(Benchmark):
    r"""Implementations of HGBat functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
        **HGBat Function**
        :math:`f(\textbf{x}) = \left| \left( \sum_{i=1}^D x_i^2 \right)^2 - \left( \sum_{i=1}^D x_i \right)^2 \right|^{\frac{1}{2}} + \frac{0.5 \sum_{i=1}^D x_i^2 + \sum_{i=1}^D x_i}{D} + 0.5`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        :math:`f(x^*) = 0`, at :math:`x^* = (420.968746,...,420.968746)`

    LaTeX formats:
        Inline:
            $$f(\textbf{x}) = \left| \left( \sum_{i=1}^D x_i^2 \right)^2 - \left( \sum_{i=1}^D x_i \right)^2 \right|^{\frac{1}{2}} + \frac{0.5 \sum_{i=1}^D x_i^2 + \sum_{i=1}^D x_i}{D} + 0.5

        Equation:
            \begin{equation} f(\textbf{x}) = \left| \left( \sum_{i=1}^D x_i^2 \right)^2 - \left( \sum_{i=1}^D x_i \right)^2 \right|^{\frac{1}{2}} + \frac{0.5 \sum_{i=1}^D x_i^2 + \sum_{i=1}^D x_i}{D} + 0.5 \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference:
        http://www5.zzu.edu.cn/__local/A/69/BC/D3B5DFE94CD2574B38AD7CD1D12_C802DAFE_BC0C0.pdf

    """

    Name = ['HGBat']

    def __init__(self, lower=-100.0, upper=100.0):
        r"""Initialize of HGBat benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = \left| \left( \sum_{i=1}^D x_i^2 \right)^2 - \left( \sum_{i=1}^D x_i \right)^2 \right|^{\frac{1}{2}} + \frac{0.5 \sum_{i=1}^D x_i^2 + \sum_{i=1}^D x_i}{D} + 0.5$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def f(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            val1, val2 = 0.0, 0.0
            for i in range(dimension):
                val1 += x[i] ** 2
            for i in range(dimension):
                val2 += x[i]
            return fabs(val1 ** 2 - val2 ** 2) ** (1 / 2) + (0.5 * val1 + val2) / dimension + 0.5

        return f

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8
# from unittest import skip
#
# from niapy.algorithms.modified import StrategyAdaptationDifferentialEvolution, StrategyAdaptationDifferentialEvolutionV1
# from niapy.tests.test_algorithm import AlgorithmTestCase, MyBenchmark
#
#
# class SADETestCase(AlgorithmTestCase):
#     def setUp(self):
#         AlgorithmTestCase.setUp(self)
#         self.algo = StrategyAdaptationDifferentialEvolution
#
#     @skip('Not implemented yet!')
#     def test_custom(self):
#         sade_custom = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
#         sade_customc = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
#         AlgorithmTestCase.test_algorithm_run(self, sade_custom, sade_customc, MyBenchmark())
#
#     @skip('Not implemented yet!')
#     def test_griewank(self):
#         sade_griewank = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
#         sade_griewankc = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
#         AlgorithmTestCase.test_algorithm_run(self, sade_griewank, sade_griewankc)
#
#
# class SADEv1TestCase(AlgorithmTestCase):
#     def setUp(self):
#         AlgorithmTestCase.setUp(self)
#         self.algo = StrategyAdaptationDifferentialEvolutionV1
#
#     @skip('Not implemented yet!')
#     def test_custom(self):
#         sadev1_custom = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
#         sadev1_customc = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
#         AlgorithmTestCase.test_algorithm_run(self, sadev1_custom, sadev1_customc, MyBenchmark())
#
#     @skip('Not implemented yet!')
#     def test_griewank(self):
#         sadev1_griewank = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
#         sadev1_griewankc = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
#         AlgorithmTestCase.test_algorithm_run(self, sadev1_griewank, sadev1_griewankc)



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm

__all__ = ['FireflyAlgorithm']

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')


class FireflyAlgorithm(Algorithm):
    r"""Implementation of Firefly algorithm.

    Algorithm:
        Firefly algorithm

    Date:
        2016

    Authors:
        Iztok Fister Jr, Iztok Fister and Klemen Berkovič

    License:
        MIT

    Reference paper:
        Fister, I., Fister Jr, I., Yang, X. S., & Brest, J. (2013).
        A comprehensive review of firefly algorithms. Swarm and Evolutionary Computation, 13, 34-46.

    Attributes:
        Name (List[str]): List of strings representing algorithm name.
        alpha (float): Step size.
        beta_min (float): Minimum value for beta.
        gamma (float): Absorption coefficient.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['FireflyAlgorithm', 'FA']

    @staticmethod
    def info():
        r"""Get algorithms information.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""Fister, I., Fister Jr, I., Yang, X. S., & Brest, J. (2013). A comprehensive review of firefly algorithms. Swarm and Evolutionary Computation, 13, 34-46."""

    def __init__(self, population_size=20, alpha=1, beta_min=1, gamma=2, *args, **kwargs):
        """Initialize FireflyAlgorithm.

        Args:
            population_size (Optional[int]): Population size.
            alpha (Optional[float]): Step size.
            beta_min (Optional[float]): Minimum value of beta.
            gamma (Optional[float]): Absorption coefficient.
            kwargs (Dict[str, Any]): Additional arguments.

        See Also:
            * :func:`niapy.algorithms.Algorithm.__init__`

        """
        super().__init__(population_size, *args, **kwargs)
        self.alpha = alpha
        self.beta_min = beta_min
        self.gamma = gamma

    def set_parameters(self, population_size=20, alpha=1, beta_min=1, gamma=2, **kwargs):
        r"""Set the parameters of the algorithm.

        Args:
            population_size (Optional[int]): Population size.
            alpha (Optional[float]): Step size.
            beta_min (Optional[float]): Minimum value of beta.
            gamma (Optional[float]): Absorption coefficient.
            kwargs (Dict[str, Any]): Additional arguments.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        Algorithm.set_parameters(self, population_size=population_size, **kwargs)
        self.alpha = alpha
        self.beta_min = beta_min
        self.gamma = gamma

    @staticmethod
    def alpha_new(a, alpha):
        r"""Optionally recalculate the new alpha value.

        Args:
            a (float):
            alpha (float):

        Returns:
            float: New value of parameter alpha.

        """
        delta = 1.0 - pow(pow(10.0, -4.0) / 0.9, 1.0 / float(a))
        return (1 - delta) * alpha

    def move_ffa(self, i, fireflies, intensity, o_fireflies, alpha, task):
        r"""Move fireflies.

        Args:
            i (int): Index of current individual.
            fireflies (numpy.ndarray):
            intensity (numpy.ndarray):
            o_fireflies (numpy.ndarray):
            alpha (float):
            task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray, bool]:
                1. New individual
                2. ``True`` if individual was moved, ``False`` if individual was not moved

        """
        moved = False
        for j in range(self.population_size):
            r = np.sum((fireflies[i] - fireflies[j]) ** 2) ** (1 / 2)
            if intensity[i] <= intensity[j]:
                continue
            beta = (1.0 - self.beta_min) * np.exp(-self.gamma * r ** 2.0) + self.beta_min
            tmp_f = alpha * (self.random(task.dimension) - 0.5) * task.range
            fireflies[i] = task.repair(fireflies[i] * (1.0 - beta) + o_fireflies[j] * beta + tmp_f, rng=self.rng)
            moved = True
        return fireflies[i], moved

    def init_population(self, task):
        r"""Initialize the starting population.

        Args:
            task (Task): Optimization task

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float], Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. Additional arguments:
                    * alpha (float): Step size.

        See Also:
            * :func:`niapy.algorithms.Algorithm.init_population`

        """
        fireflies, intensity, _ = Algorithm.init_population(self, task)
        return fireflies, intensity, {'alpha': self.alpha}

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of Firefly Algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray): Current population function/fitness values.
            best_x (numpy.ndarray): Global best individual.
            best_fitness (float): Global best individual fitness/function value.
            **params (Dict[str, Any]): Additional arguments.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. New global best solution
                4. New global best solutions fitness/objective value
                5. Additional arguments:
                    * alpha (float): Step size.

        See Also:
            * :func:`niapy.algorithms.basic.FireflyAlgorithm.move_ffa`

        """
        alpha = params.pop('alpha')

        alpha = self.alpha_new(task.max_evals / self.population_size, alpha)
        sorted_index = np.argsort(population_fitness)
        tmp = np.asarray(
            [self.move_ffa(i, population[sorted_index], population_fitness[sorted_index],
                           population, alpha, task) for i in range(self.population_size)], dtype=object)
        population = np.asarray([tmp[i][0] for i in range(len(tmp))], dtype=object)
        moved = np.asarray([tmp[i][1] for i in range(len(tmp))])
        population_fitness[np.where(moved)] = np.apply_along_axis(task.eval, 1, population[np.where(moved)])
        best_x, best_fitness = self.get_best(population, population_fitness, best_x, best_fitness)
        return population, population_fitness, best_x, best_fitness, {'alpha': alpha}

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8

"""Implementation of Happy Cat function."""

import math
from niapy.benchmarks.benchmark import Benchmark

__all__ = ['HappyCat']


class HappyCat(Benchmark):
    r"""Implementation of Happy cat function.

    Date: 2018

    Author: Lucija Brezočnik

    License: MIT

    Function: **Happy cat function**

        :math:`f(\mathbf{x}) = {\left |\sum_{i = 1}^D {x_i}^2 - D \right|}^{1/4} + (0.5 \sum_{i = 1}^D {x_i}^2 + \sum_{i = 1}^D x_i) / D + 0.5`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (-1,...,-1)`

    LaTeX formats:
        Inline:
            $f(\mathbf{x}) = {\left|\sum_{i = 1}^D {x_i}^2 -
            D \right|}^{1/4} + (0.5 \sum_{i = 1}^D {x_i}^2 +
            \sum_{i = 1}^D x_i) / D + 0.5$

        Equation:
            \begin{equation} f(\mathbf{x}) = {\left| \sum_{i = 1}^D {x_i}^2 -
            D \right|}^{1/4} + (0.5 \sum_{i = 1}^D {x_i}^2 +
            \sum_{i = 1}^D x_i) / D + 0.5 \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference: http://bee22.com/manual/tf_images/Liang%20CEC2014.pdf &
    Beyer, H. G., & Finck, S. (2012). HappyCat - A Simple Function Class Where Well-Known Direct Search Algorithms Do Fail.
    In International Conference on Parallel Problem Solving from Nature (pp. 367-376). Springer, Berlin, Heidelberg.

    """

    Name = ['HappyCat']

    def __init__(self, lower=-100.0, upper=100.0):
        r"""Initialize of Happy cat benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\mathbf{x}) = {\left|\sum_{i = 1}^D {x_i}^2 -
                D \right|}^{1/4} + (0.5 \sum_{i = 1}^D {x_i}^2 +
                \sum_{i = 1}^D x_i) / D + 0.5$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def evaluate(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            val1 = 0.0
            val2 = 0.0
            alpha = 0.125

            for i in range(dimension):
                val1 += math.pow(abs(math.pow(x[i], 2) - dimension), alpha)
                val2 += (0.5 * math.pow(x[i], 2) + x[i]) / dimension

            return val1 + val2 + 0.5

        return evaluate



# encoding=utf8
from unittest import TestCase

import niapy
from niapy.benchmarks import Benchmark


class MyBenchmark(Benchmark):
    def __init__(self):
        Benchmark.__init__(self, -11, 11)

    @classmethod
    def function(cls):
        def evaluate(d, sol):
            val = 0.0
            for i in range(d):
                val = val + sol[i] * sol[i]
            return val

        return evaluate


class RunnerTestCase(TestCase):
    def setUp(self):
        self.algorithms = ['DifferentialEvolution', 'GreyWolfOptimizer', 'GeneticAlgorithm']
        self.benchmarks = ['griewank', MyBenchmark()]

    def test_runner(self):
        self.assertTrue(niapy.Runner(7, 100, 2, self.algorithms, self.benchmarks).run())

    def test_runner_bad_algorithm_throws(self):
        self.assertRaises(KeyError, lambda: niapy.Runner(4, 10, 3, ['EvolutionStrategy'], self.benchmarks).run())

    def test_runner_bad_benchmark_throws(self):
        self.assertRaises(KeyError, lambda: niapy.Runner(4, 10, 3, ['EvolutionStrategy1p1'], ['TesterMan']).run())

    def test_runner_bad_export_throws(self):
        self.assertRaises(TypeError,
                          lambda: niapy.Runner(4, 10, 3, ['GreyWolfOptimizer'], self.benchmarks).run(export="pandas"))

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8
# TODO Not implemented, commenting out
# import logging
#
# from niapy.algorithms.algorithm import Individual
# from niapy.algorithms.basic.de import \
#     DifferentialEvolution  # , cross_best1, CrossRand1, cross_curr2best1, cross_best2, cross_curr2rand1, proportional
#
# logging.basicConfig()
# logger = logging.getLogger('niapy.algorithms.modified')
# logger.setLevel('INFO')
#
# __all__ = [
#     'StrategyAdaptationDifferentialEvolution',
#     'StrategyAdaptationDifferentialEvolutionV1'
# ]
#
#
# class StrategyAdaptationDifferentialEvolution(DifferentialEvolution):
#     r"""Implementation of Differential Evolution Algorithm With Strategy Adaptation algorithm.
#
#     Algorithm:
#         Differential Evolution Algorithm With StrategyAdaptation
#
#     Date:
#         2019
#
#     Author:
#         Klemen Berkovič
#
#     License:
#         MIT
#
#     Reference URL:
#         https://ieeexplore.ieee.org/document/1554904
#
#     Reference paper:
#         Qin, A. Kai, and Ponnuthurai N. Suganthan. "Self-adaptive differential evolution algorithm for numerical optimization." 2005 IEEE congress on evolutionary computation. Vol. 2. IEEE, 2005.
#
#     Attributes:
#         Name (List[str]): List of strings representing algorithm name.
#
#     See Also:
#         :class:`niapy.algorithms.basic.DifferentialEvolution`
#     """
#     Name = ['StrategyAdaptationDifferentialEvolution', 'SADE', 'SaDE']
#
#     @staticmethod
#     def info():
#         r"""Get basic information about the algorithm.
#
#         Returns:
#             str: Basic information.
#
#         See Also:
#             :func:`niapy.algorithms.algorithm.Algorithm.info`
#         """
#         return r"""Qin, A. Kai, and Ponnuthurai N. Suganthan. "Self-adaptive differential evolution algorithm for numerical optimization." 2005 IEEE congress on evolutionary computation. Vol. 2. IEEE, 2005."""
#
#     def set_parameters(self, **kwargs):
#         DifferentialEvolution.set_parameters(self, **kwargs)
#
#
#
#     def get_parameters(self):
#         d = DifferentialEvolution.get_parameters(self)
#
#         return d
#
#     def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
#
#         return population, population_fitness, best_x, best_fitness, params
#
#
# class StrategyAdaptationDifferentialEvolutionV1(DifferentialEvolution):
#     r"""Implementation of Differential Evolution Algorithm With Strategy Adaptation algorithm.
#
#     Algorithm:
#         Differential Evolution Algorithm With StrategyAdaptation
#
#     Date:
#         2019
#
#     Author:
#         Klemen Berkovič
#
#     License:
#         MIT
#
#     Reference URL:
#         https://ieeexplore.ieee.org/document/4632146
#
#     Reference paper:
#         Qin, A. Kai, Vicky Ling Huang, and Ponnuthurai N. Suganthan. "Differential evolution algorithm with strategy adaptation for global numerical optimization." IEEE transactions on Evolutionary Computation 13.2 (2009): 398-417.
#
#     Attributes:
#         Name (List[str]): List of strings representing algorithm name.
#
#     See Also:
#         :class:`niapy.algorithms.basic.DifferentialEvolution`
#     """
#     Name = ['StrategyAdaptationDifferentialEvolutionV1', 'SADEV1', 'SaDEV1']
#
#     @staticmethod
#     def info():
#         r"""Get basic information about the algorithm.
#
#         Returns:
#             str: Basic information.
#
#         See Also:
#             :func:`niapy.algorithms.algorithm.Algorithm.info`
#         """
#         return r"""Qin, A. Kai, Vicky Ling Huang, and Ponnuthurai N. Suganthan. "Differential evolution algorithm with strategy adaptation for global numerical optimization." IEEE transactions on Evolutionary Computation 13.2 (2009): 398-417."""
#
#     def set_parameters(self, **kwargs):
#         DifferentialEvolution.set_parameters(self, **kwargs)
#
#
#
#     def get_parameters(self):
#         d = DifferentialEvolution.get_parameters(self)
#         return d
#
#     def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
#         return population, population_fitness, best_x, best_fitness, params

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8

"""Implementations of Michalewichz's function."""

import numpy as np
from niapy.benchmarks.benchmark import Benchmark

__all__ = ['Michalewichz']


class Michalewichz(Benchmark):
    r"""Implementations of Michalewichz's functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **High Conditioned Elliptic Function**

        :math:`f(\textbf{x}) = \sum_{i=1}^D \left( 10^6 \right)^{ \frac{i - 1}{D - 1} } x_i^2`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [0, \pi]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        at :math:`d = 2` :math:`f(\textbf{x}^*) = -1.8013` at :math:`\textbf{x}^* = (2.20, 1.57)`
        at :math:`d = 5` :math:`f(\textbf{x}^*) = -4.687658`
        at :math:`d = 10` :math:`f(\textbf{x}^*) = -9.66015`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = - \sum_{i = 1}^{D} \sin(x_i) \sin\left( \frac{ix_i^2}{\pi} \right)^{2m}$

        Equation:
            \begin{equation} f(\textbf{x}) = - \sum_{i = 1}^{D} \sin(x_i) \sin\left( \frac{ix_i^2}{\pi} \right)^{2m} \end{equation}

        Domain:
            $0 \leq x_i \leq \pi$

    Reference URL:
        https://www.sfu.ca/~ssurjano/michal.html

    """

    Name = ['Michalewichz']

    def __init__(self, lower=0.0, upper=np.pi, m=10):
        r"""Initialize of Michalewichz benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.
            m (float): Steepness of valleys and ridges. Recommended value is 10.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)
        self.m = m

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = - \sum_{i = 1}^{D} \sin(x_i) \sin\left( \frac{ix_i^2}{\pi} \right)^{2m}$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def evaluate(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            v = 0.0
            for i in range(dimension):
                v += np.sin(x[i]) * np.sin(((i + 1) * x[i] ** 2) / np.pi) ** (2 * self.m)
            return -v

        return evaluate

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8

"""Implementations of Perm function."""

from niapy.benchmarks.benchmark import Benchmark

__all__ = ['Perm']


class Perm(Benchmark):
    r"""Implementations of Perm functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Perm Function**

        :math:`f(\textbf{x}) = \sum_{i = 1}^D \left( \sum_{j = 1}^D (j - \beta) \left( x_j^i - \frac{1}{j^i} \right) \right)^2`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-D, D]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        :math:`f(\textbf{x}^*) = 0` at :math:`\textbf{x}^* = (1, \frac{1}{2}, \cdots , \frac{1}{i} , \cdots , \frac{1}{D})`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = \sum_{i = 1}^D \left( \sum_{j = 1}^D (j - \beta) \left( x_j^i - \frac{1}{j^i} \right) \right)^2$

        Equation:
            \begin{equation} f(\textbf{x}) = \sum_{i = 1}^D \left( \sum_{j = 1}^D (j - \beta) \left( x_j^i - \frac{1}{j^i} \right) \right)^2 \end{equation}

        Domain:
            $-D \leq x_i \leq D$

    Reference:
        https://www.sfu.ca/~ssurjano/perm0db.html


    """

    Name = ['Perm']

    def __init__(self, dimension=10.0, beta=.5):
        r"""Initialize of Bent Cigar benchmark.

        Args:
            dimension (float): Dimension of the problem.
                -dimension will be the lower bound, dimension will be the upper bound.
            beta (float): Beta parameter.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(-dimension, dimension)
        self.beta = beta

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = \sum_{i = 1}^D \left( \sum_{j = 1}^D (j - \beta) \left( x_j^i - \frac{1}{j^i} \right) \right)^2$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def f(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            v = .0
            for i in range(1, dimension + 1):
                vv = .0
                for j in range(1, dimension + 1):
                    vv += (j + self.beta) * (x[j - 1] ** i - 1 / j ** i)
                v += vv ** 2
            return v

        return f

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8

"""The implementation of tasks."""

import logging
from enum import Enum

import numpy as np
from matplotlib import pyplot as plt
from niapy.benchmarks import Benchmark
from niapy.util.array import full_array
from niapy.util.repair import limit
from niapy.util.factory import get_benchmark
from niapy.util.exception import FesException, GenException, RefException

logging.basicConfig()
logger = logging.getLogger("niapy.task.Task")
logger.setLevel("INFO")


class OptimizationType(Enum):
    r"""Enum representing type of optimization.

    Attributes:
        MINIMIZATION (int): Represents minimization problems and is default optimization type of all algorithms.
        MAXIMIZATION (int): Represents maximization problems.

    """

    MINIMIZATION = 1.0
    MAXIMIZATION = -1.0


class Task:
    r"""Class representing problem to solve with optimization.

    Date:
        2019

    Author:
        Klemen Berkovič and others

    Attributes:
        dimension (int): Dimension of the problem.
        lower (numpy.ndarray): Lower bounds of the problem.
        upper (numpy.ndarray): Upper bounds of the problem.
        range (numpy.ndarray): Search range between upper and lower limits.
        optimization_type (OptimizationType): Optimization type to use.

    """

    def __init__(self, dimension=0, optimization_type=OptimizationType.MINIMIZATION, benchmark=None, lower=None,
                 upper=None, repair_function=limit):
        r"""Initialize task class for optimization.

        Args:
            dimension (Optional[int]): Number of dimensions.
            optimization_type (Optional[OptimizationType]): Set the type of optimization.
            benchmark (Union[str, Benchmark]): Problem to solve with optimization.
            lower (Optional[numpy.ndarray]): Lower limits of the problem.
            upper (Optional[numpy.ndarray]): Upper limits of the problem.
            repair_function (Optional[Callable[[numpy.ndarray, numpy.ndarray, numpy.ndarray, Dict[str, Any]], numpy.ndarray]]): Function for repairing individuals components to desired limits.

        See Also:
            * `func`:niapy.task.Utility.__init__`

        """
        # dimension of the problem
        self.dimension = dimension
        # set optimization type
        self.optimization_type = optimization_type
        # set optimization function
        self.benchmark = None
        if isinstance(benchmark, str):
            self.benchmark = get_benchmark(benchmark)
        elif isinstance(benchmark, Benchmark):
            self.benchmark = benchmark

        if self.benchmark is not None:
            self.function = self.benchmark.function() if self.benchmark is not None else None

        # set Lower limits
        if lower is not None:
            self.lower = full_array(lower, self.dimension)
        elif lower is None and benchmark is not None:
            self.lower = full_array(self.benchmark.lower, self.dimension)
        else:
            self.lower = full_array(0, self.dimension)

        # set Upper limits
        if upper is not None:
            self.upper = full_array(upper, self.dimension)
        elif upper is None and benchmark is not None:
            self.upper = full_array(self.benchmark.upper, self.dimension)
        else:
            self.upper = full_array(0, self.dimension)

        # set range
        self.range = self.upper - self.lower
        # set repair function
        self.repair_function = repair_function

    def repair(self, x, rng=None):
        r"""Repair solution and put the solution in the random position inside of the bounds of problem.

        Args:
            x (numpy.ndarray): Solution to check and repair if needed.
            rng (numpy.random.Generator): Random number generator.

        Returns:
            numpy.ndarray: Fixed solution.

        See Also:
            * :func:`niapy.util.repair.limit`
            * :func:`niapy.util.repair.limit_inverse`
            * :func:`niapy.util.repair.wang`
            * :func:`niapy.util.repair.rand`
            * :func:`niapy.util.repair.reflect`

        """
        return self.repair_function(x, self.lower, self.upper, rng=rng)

    def next_iter(self):
        r"""Increments the number of algorithm iterations."""

    def start(self):
        r"""Start stopwatch."""

    def eval(self, x):
        r"""Evaluate the solution A.

        Args:
            x (numpy.ndarray): Solution to evaluate.

        Returns:
            float: Fitness/function values of solution.

        """
        return self.function(self.dimension, x) * self.optimization_type.value

    def is_feasible(self, x):
        r"""Check if the solution is feasible.

        Args:
            x (Union[numpy.ndarray, Individual]): Solution to check for feasibility.

        Returns:
            bool: `True` if solution is in feasible space else `False`.

        """
        return np.all((x >= self.lower) & (x <= self.upper))

    def stopping_condition(self):
        r"""Check if optimization task should stop.

        Returns:
            bool: `True` if stopping condition is meet else `False`.

        """
        return False


class CountingTask(Task):
    r"""Optimization task with added counting of function evaluations and algorithm iterations/generations.

    Attributes:
        iters (int): Number of algorithm iterations/generations.
        evals (int): Number of function evaluations.

    See Also:
        * :class:`niapy.task.Task`

    """

    def __init__(self, dimension=0, optimization_type=OptimizationType.MINIMIZATION, benchmark=None, lower=None,
                 upper=None, repair_function=limit):
        r"""Initialize counting task.

        Args:
            dimension (Optional[int]): Number of dimensions.
            optimization_type (Optional[OptimizationType]): Set the type of optimization.
            benchmark (Union[str, Benchmark]): Problem to solve with optimization.
            lower (Optional[numpy.ndarray]): Lower limits of the problem.
            upper (Optional[numpy.ndarray]): Upper limits of the problem.
            repair_function (Optional[Callable[[numpy.ndarray, numpy.ndarray, numpy.ndarray, Dict[str, Any]], numpy.ndarray]]): Function for repairing individuals components to desired limits.

        See Also:
            * :func:`niapy.task.Task.__init__`

        """
        super().__init__(dimension, optimization_type, benchmark, lower, upper, repair_function)
        self.iters = 0
        self.evals = 0

    def eval(self, x):
        r"""Evaluate the solution x.

        This function increments function evaluation counter `self.Evals`.

        Args:
            x (numpy.ndarray): Solutions to evaluate.

        Returns:
            float: Fitness/function values of solution.

        See Also:
            * :func:`niapy.task.Task.eval`

        """
        r = super().eval(x)
        self.evals += 1
        return r

    def next_iter(self):
        r"""Increases the number of algorithm iterations made.

        This function increments number of algorithm iterations/generations counter `self.Iters`.

        """
        self.iters += 1


class StoppingTask(CountingTask):
    r"""Optimization task with implemented checking for stopping criteria.

    Attributes:
        max_iters (int): Maximum number of algorithm iterations/generations.
        max_evals (int): Maximum number of function evaluations.
        cutoff_value (float): Reference function/fitness values to reach in optimization.
        x (numpy.ndarray): Best found individual.
        x_f (float): Best found individual function/fitness value.

    See Also:
        * :class:`niapy.task.CountingTask`

    """

    def __init__(self, max_evals=np.inf, max_iters=np.inf, cutoff_value=None, enable_logging=False, dimension=0,
                 optimization_type=OptimizationType.MINIMIZATION, benchmark=None, lower=None, upper=None,
                 repair_function=limit):
        r"""Initialize task class for optimization.

        Args:
            max_evals (Optional[int]): Number of function evaluations.
            max_iters (Optional[int]): Number of generations or iterations.
            cutoff_value (Optional[float]): Reference value of function/fitness function.
            enable_logging (Optional[bool]): Enable/disable logging of improvements.
            dimension (Optional[int]): Number of dimensions.
            optimization_type (Optional[OptimizationType]): Set the type of optimization.
            benchmark (Union[str, Benchmark]): Problem to solve with optimization.
            lower (Optional[numpy.ndarray]): Lower limits of the problem.
            upper (Optional[numpy.ndarray]): Upper limits of the problem.
            repair_function (Optional[Callable[[numpy.ndarray, numpy.ndarray, numpy.ndarray, Dict[str, Any]], numpy.ndarray]]): Function for repairing individuals components to desired limits.

        Note:
            Storing improvements during the evolutionary cycle is
            captured in self.n_evals and self.x_f_vals

        See Also:
            * :func:`niapy.task.CountingTask.__init__`

        """
        super().__init__(dimension, optimization_type, benchmark, lower, upper, repair_function)
        self.cutoff_value = -np.inf * optimization_type.value if cutoff_value is None else cutoff_value
        self.enable_logging = enable_logging
        self.x = None
        self.x_f = np.inf * optimization_type.value
        self.max_evals = max_evals
        self.max_iters = max_iters
        self.n_evals = []
        self.x_f_vals = []

    def eval(self, x):
        r"""Evaluate solution.

        Args:
            x (numpy.ndarray): Solution to evaluate.

        Returns:
            float: Fitness/function value of solution.

        See Also:
            * :func:`niapy.task.StoppingTask.stopping_condition`
            * :func:`niapy.task.CountingTask.eval`

        """
        if self.stopping_condition():
            return np.inf * self.optimization_type.value

        x_f = super().eval(x)

        if x_f < self.x_f:
            self.x_f = x_f
            self.n_evals.append(self.evals)
            self.x_f_vals.append(x_f)
            if self.enable_logging:
                logger.info('evals:%d => %s' % (self.evals, self.x_f))

        return x_f

    def stopping_condition(self):
        r"""Check if stopping condition reached.

        Returns:
            bool: `True` if number of function evaluations or number of algorithm iterations/generations or reference values is reach else `False`.

        """
        return (self.evals >= self.max_evals) or (self.iters >= self.max_iters) or (self.cutoff_value > self.x_f)

    def stopping_condition_iter(self):
        r"""Check if stopping condition reached and increase number of iterations.

        Returns:
            bool: `True` if number of function evaluations or number of algorithm iterations/generations or reference values is reach else `False`.

        See Also:
            * :func:`niapy.task.StoppingTask.stopping_condition`
            * :func:`niapy.task.CountingTask.next_iter`

        """
        r = self.stopping_condition()
        self.next_iter()
        return r

    def return_conv(self):
        r"""Get values of x and y axis for plotting covariance graph.

        Returns:
            Tuple[List[int], List[float]]:
                1. List of ints of function evaluations.
                2. List of ints of function/fitness values.

        """
        r1, r2 = [], []
        for i, v in enumerate(self.n_evals):
            r1.append(v), r2.append(self.x_f_vals[i])
            if i >= len(self.n_evals) - 1:
                break
            diff = self.n_evals[i + 1] - v
            if diff <= 1:
                continue
            for j in range(diff - 1):
                r1.append(v + j + 1), r2.append(self.x_f_vals[i])
        return r1, r2

    def plot(self):
        """Plot a simple convergence graph."""
        fess, fitnesses = self.return_conv()
        plt.plot(fess, fitnesses)
        plt.xlabel('nFes')
        plt.ylabel('Fitness')
        plt.title('Convergence graph')
        plt.show()


class ThrowingTask(StoppingTask):
    r"""Task that throw exceptions when stopping condition is meet.

    See Also:
        * :class:`niapy.task.StoppingTask`

    """

    def stopping_condition_throw(self):
        r"""Throw exception for the given stopping condition.

        Raises:
            * FesException: Thrown when the number of function/fitness evaluations is reached.
            * GenException: Thrown when the number of algorithms generations/iterations is reached.
            * RefException: Thrown when the reference values is reached.
            * TimeException: Thrown when algorithm exceeds time run limit.

        """
        if self.evals >= self.max_evals:
            raise FesException()
        if self.iters >= self.max_iters:
            raise GenException()
        if self.cutoff_value >= self.x_f:
            raise RefException()

    def eval(self, x):
        r"""Evaluate solution.

        Args:
            x (numpy.ndarray): Solution to evaluate.

        Returns:
            float: Function/fitness values of solution.

        See Also:
            * :func:`niapy.task.ThrowingTask.stopping_condition_throw`
            * :func:`niapy.task.StoppingTask.eval`

        """
        self.stopping_condition_throw()
        return super().eval(x)



# encoding=utf8

"""Implementation of benchmarks utility function."""

import logging
import numpy as np
from matplotlib import pyplot as plt
from matplotlib import cm
from mpl_toolkits.mplot3d import Axes3D

logging.basicConfig()
logger = logging.getLogger('niapy.benchmarks.benchmark')
logger.setLevel('INFO')

__all__ = ['Benchmark']


class Benchmark:
    r"""Class representing benchmarks.

    Attributes:
        Name (List[str]): List of names representing benchmark names.
        lower (Union[int, float, list, numpy.ndarray]): Lower bounds.
        upper (Union[int, float, list, numpy.ndarray]): Upper bounds.

    """

    Name = ['Benchmark', 'BBB']

    def __init__(self, lower, upper):
        r"""Initialize benchmark.

        Args:
            lower (Union[int, float, list, numpy.ndarray]): Lower bounds.
            upper (Union[int, float, list, numpy.ndarray]): Upper bounds.

        """
        self.lower = lower
        self.upper = upper

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code

        """
        return r'''$f(x) = \infty$'''

    def function(self):
        r"""Get the optimization function.

        Returns:
            Callable[[int, Union[list, numpy.ndarray]], float]: Fitness function.

        """
        def fun(dimension, x):
            r"""Initialize benchmark.

            Args:
                dimension (int): Dimensionality of the problem.
                x (Union[int, float, list, numpy.ndarray]): Solution to the problem.

            Returns:
                float: Fitness value for the solution.

            """
            return np.inf

        return fun

    def __call__(self):
        r"""Get the optimization function.

        Returns:
            Callable[[int, Union[list, numpy.ndarray]], float]: Fitness function.

        """
        return self.function()

    def plot2d(self):
        r"""Plot 2D graph."""
        pass

    @staticmethod
    def __2d_fun(x, y, f):
        r"""Calculate function value.

        Args:
            x (float): First coordinate.
            y (float): Second coordinate.
            f (Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]): Evaluation function.

        Returns:
            float: Calculate functional value for given input.

        """
        return f(2, [x, y])

    def plot3d(self, scale=0.32):
        r"""Plot 3d scatter plot of benchmark function.

        Args:
            scale (float): Scale factor for points.

        """
        fig = plt.figure()
        ax = Axes3D(fig)
        func = self.function()
        xr, yr = np.arange(self.lower, self.upper, scale), np.arange(self.lower, self.upper, scale)
        x, y = np.meshgrid(xr, yr)
        z = np.vectorize(self.__2d_fun)(x, y, func)
        ax.plot_surface(x, y, z, rstride=8, cstride=8, alpha=0.3)
        ax.contourf(x, y, z, zdir='z', offset=-10, cmap=cm.coolwarm)
        ax.set_xlabel('X')
        ax.set_ylabel('Y')
        ax.set_zlabel('Z')
        plt.show()

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8


"""Implementation of Qing funcion."""

import math
from niapy.benchmarks.benchmark import Benchmark

__all__ = ['Qing']


class Qing(Benchmark):
    r"""Implementation of Qing function.

    Date: 2018

    Author: Lucija Brezočnik

    License: MIT

    Function: **Qing function**

        :math:`f(\mathbf{x}) = \sum_{i=1}^D \left(x_i^2 - i\right)^2`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-500, 500]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (\pm √i))`

    LaTeX formats:
        Inline:
            $f(\mathbf{x}) = \sum_{i=1}^D \left (x_i^2 - i\right)^2$

        Equation:
            \begin{equation} f(\mathbf{x}) =
            \sum_{i=1}^D \left{(x_i^2 - i\right)}^2 \end{equation}

        Domain:
            $-500 \leq x_i \leq 500$

    Reference paper:
        Jamil, M., and Yang, X. S. (2013).
        A literature survey of benchmark functions for global optimisation problems.
        International Journal of Mathematical Modelling and Numerical Optimisation,
        4(2), 150-194.

    """

    Name = ['Qing']

    def __init__(self, lower=-500.0, upper=500.0):
        r"""Initialize of Qing benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\mathbf{x}) = \sum_{i=1}^D \left (x_i^2 - i\right)^2$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def evaluate(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            val = 0.0

            for i in range(dimension):
                val += math.pow(math.pow(x[i], 2) - i, 2)

            return val

        return evaluate



# encoding=utf8

"""Implementation of Ridge function."""

import math
from niapy.benchmarks.benchmark import Benchmark

__all__ = ['Ridge']


class Ridge(Benchmark):
    r"""Implementation of Ridge function.

    Date: 2018

    Author: Lucija Brezočnik

    License: MIT

    Function: **Ridge function**

        :math:`f(\mathbf{x}) = \sum_{i=1}^D (\sum_{j=1}^i x_j)^2`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-64, 64]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (0,...,0)`

    LaTeX formats:
        Inline:
            $f(\mathbf{x}) = \sum_{i=1}^D (\sum_{j=1}^i x_j)^2 $

        Equation:
            \begin{equation} f(\mathbf{x}) =
            \sum_{i=1}^D (\sum_{j=1}^i x_j)^2 \end{equation}

        Domain:
            $-64 \leq x_i \leq 64$

    Reference:
        http://www.cs.unm.edu/~neal.holts/dga/benchmarkFunction/ridge.html

    """

    Name = ['Ridge']

    def __init__(self, lower=-64.0, upper=64.0):
        r"""Initialize of Ridge benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\mathbf{x}) = \sum_{i=1}^D (\sum_{j=1}^i x_j)^2 $'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def evaluate(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            val = 0.0

            for i in range(dimension):
                val1 = 0.0
                for j in range(i + 1):
                    val1 += x[j]
                val += math.pow(val1, 2)

            return val

        return evaluate



# encoding=utf8
# from unittest import TestCase
#
# import numpy as np
# from numpy.random import default_rng
#
# from niapy.algorithms.modified import AdaptiveArchiveDifferentialEvolution, cross_curr2p_best
# from niapy.tests.test_algorithm import AlgorithmTestCase, MyBenchmark
# from niapy.util import full_array
#
#
# class CrossRandCurr2pbestTestCase(TestCase):
#     def setUp(self):
#         self.D, self.NP, self.F, self.CR, self.p = 10, 100, 0.5, 0.5, 0.25
#         self.Upper, self.Lower = full_array(100, self.D), full_array(-100, self.D)
#         self.evalFun = MyBenchmark().function()
#
#     def init_pop(self):
#         pop = self.Lower + default_rng().random((self.NP, self.D)) * (self.Upper - self.Lower)
#         return pop, np.asarray([self.evalFun(self.D, x) for x in pop])
#
#     def test_function(self):
#         pop, fpop = self.init_pop()
#         apop, _ = self.init_pop()
#         ib = np.argmin(fpop)
#         xb, fxb = pop[ib].copy(), fpop[ib]
#         for i, x in enumerate(pop):
#             xn = cross_curr2p_best(pop, i, xb, self.F, self.CR, default_rng(), self.p, apop)
#             # Check that position array is updated
#             self.assertFalse(np.array_equal(x, xn))
#
#
# class JADETestCase(AlgorithmTestCase):
#     def setUp(self):
#         AlgorithmTestCase.setUp(self)
#         self.algo = AdaptiveArchiveDifferentialEvolution
#
#     def test_custom(self):
#         jade_custom = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
#         jade_customc = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
#         AlgorithmTestCase.test_algorithm_run(self, jade_custom, jade_customc, MyBenchmark())
#
#     def test_griewank(self):
#         jade_griewank = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
#         jade_griewankc = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
#         AlgorithmTestCase.test_algorithm_run(self, jade_griewank, jade_griewankc)

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8
import logging
from unittest import TestCase

import numpy as np

from niapy.benchmarks import Benchmark

logging.basicConfig()
logger = logging.getLogger('niapy.test')
logger.setLevel('INFO')


class BenchmarkTestCase(TestCase):
    def setUp(self):
        self.Lower, self.Upper = [-19, -10], [19, 5]
        self.bc = Benchmark
        self.b = self.bc(self.Lower, self.Upper)

    def test_lower(self):
        self.assertEqual(self.Lower, self.b.lower)

    def test_upper(self):
        self.assertEqual(self.Upper, self.b.upper)

    def test_function_eval(self):
        f = self.b.function()
        self.assertTrue(callable(f))
        self.assertEqual(np.inf, f(1, self.Upper))

    def test_latex_code(self):
        info = self.bc.latex_code()
        self.assertIsNotNone(info)

    def test_call_operator(self):
        f = self.b()
        self.assertIsNotNone(f)

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8

"""Implementation of Griewank funcion."""

from math import sqrt, cos
from niapy.benchmarks.benchmark import Benchmark

__all__ = ['Griewank', 'ExpandedGriewankPlusRosenbrock']


class Griewank(Benchmark):
    r"""Implementation of Griewank function.

    Date: 2018

    Authors: Iztok Fister Jr. and Lucija Brezočnik

    License: MIT

    Function: **Griewank function**

        :math:`f(\mathbf{x}) = \sum_{i=1}^D \frac{x_i^2}{4000} - \prod_{i=1}^D \cos(\frac{x_i}{\sqrt{i}}) + 1`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (0,...,0)`

    LaTeX formats:
        Inline:
            $f(\mathbf{x}) = \sum_{i=1}^D \frac{x_i^2}{4000} -
            \prod_{i=1}^D \cos(\frac{x_i}{\sqrt{i}}) + 1$

        Equation:
            \begin{equation} f(\mathbf{x}) = \sum_{i=1}^D \frac{x_i^2}{4000} -
            \prod_{i=1}^D \cos(\frac{x_i}{\sqrt{i}}) + 1 \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference paper:
    Jamil, M., and Yang, X. S. (2013).
    A literature survey of benchmark functions for global optimisation problems.
    International Journal of Mathematical Modelling and Numerical Optimisation,
    4(2), 150-194.

    """

    Name = ['Griewank']

    def __init__(self, lower=-100.0, upper=100.0):
        r"""Initialize of Griewank benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\mathbf{x}) = \sum_{i=1}^D \frac{x_i^2}{4000} - \prod_{i=1}^D \cos(\frac{x_i}{\sqrt{i}}) + 1$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def evaluate(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            val1, val2 = 0.0, 1.0
            for i in range(dimension):
                val1 += x[i] ** 2 / 4000.0
                val2 *= cos(x[i] / sqrt(i + 1))
            return val1 - val2 + 1.0

        return evaluate


class ExpandedGriewankPlusRosenbrock(Benchmark):
    r"""Implementation of Expanded Griewank's plus Rosenbrock function.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function: **Expanded Griewank's plus Rosenbrock function**

        :math:`f(\textbf{x}) = h(g(x_D, x_1)) + \sum_{i=2}^D h(g(x_{i - 1}, x_i)) \\ g(x, y) = 100 (x^2 - y)^2 + (x - 1)^2 \\ h(z) = \frac{z^2}{4000} - \cos \left( \frac{z}{\sqrt{1}} \right) + 1`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-100, 100]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        :math:`f(x^*) = 0`, at :math:`x^* = (420.968746,...,420.968746)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = h(g(x_D, x_1)) + \sum_{i=2}^D h(g(x_{i - 1}, x_i)) \\ g(x, y) = 100 (x^2 - y)^2 + (x - 1)^2 \\ h(z) = \frac{z^2}{4000} - \cos \left( \frac{z}{\sqrt{1}} \right) + 1$

        Equation:
            \begin{equation} f(\textbf{x}) = h(g(x_D, x_1)) + \sum_{i=2}^D h(g(x_{i - 1}, x_i)) \\ g(x, y) = 100 (x^2 - y)^2 + (x - 1)^2 \\ h(z) = \frac{z^2}{4000} - \cos \left( \frac{z}{\sqrt{1}} \right) + 1 \end{equation}

        Domain:
            $-100 \leq x_i \leq 100$

    Reference:
        http://www5.zzu.edu.cn/__local/A/69/BC/D3B5DFE94CD2574B38AD7CD1D12_C802DAFE_BC0C0.pdf

    """

    Name = ['ExpandedGriewankPlusRosenbrock']

    def __init__(self, lower=-100.0, upper=100.0):
        r"""Initialize of Expanded Griewank's plus Rosenbrock benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = h(g(x_D, x_1)) + \sum_{i=2}^D h(g(x_{i - 1}, x_i)) \\ g(x, y) = 100 (x^2 - y)^2 + (x - 1)^2 \\ h(z) = \frac{z^2}{4000} - \cos \left( \frac{z}{\sqrt{1}} \right) + 1$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def h(z):
            return z ** 2 / 4000 - cos(z / sqrt(1)) + 1

        def g(x, y):
            return 100 * (x ** 2 - y ** 2) ** 2 + (x - 1) ** 2

        def f(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            val = 0.0
            for i in range(1, dimension):
                val += h(g(x[i - 1], x[i]))
            return h(g(x[dimension - 1], x[0])) + val

        return f

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8
from niapy.algorithms.basic import KrillHerdV1, KrillHerdV2, KrillHerdV3, KrillHerdV4, KrillHerdV11
from niapy.tests.test_algorithm import AlgorithmTestCase, MyBenchmark


class KHV1TestCase(AlgorithmTestCase):
    r"""Test case for KrillHerdV1 algorithm.

    Date:
        April 2019

    Author:
        Klemen Berkovič

    See Also:
        * :class:`niapy.tests.test_algorithm.AlgorithmTestCase`
    """

    def setUp(self):
        AlgorithmTestCase.setUp(self)
        self.algo = KrillHerdV1

    def test_custom(self):
        kh_custom = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
        kh_customc = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
        AlgorithmTestCase.test_algorithm_run(self, kh_custom, kh_customc, MyBenchmark())

    def test_griewank(self):
        kh_griewank = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
        kh_griewankc = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
        AlgorithmTestCase.test_algorithm_run(self, kh_griewank, kh_griewankc)


class KHV2TestCase(AlgorithmTestCase):
    r"""Test case for KrillHerdV2 algorithm.

    Date:
        April 2019

    Author:
        Klemen Berkovič

    See Also:
        * :class:`niapy.tests.test_algorithm.AlgorithmTestCase`
    """

    def setUp(self):
        AlgorithmTestCase.setUp(self)
        self.algo = KrillHerdV2

    def test_custom(self):
        kh_custom = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
        kh_customc = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
        AlgorithmTestCase.test_algorithm_run(self, kh_custom, kh_customc, MyBenchmark())

    def test_griewank(self):
        kh_griewank = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
        kh_griewankc = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
        AlgorithmTestCase.test_algorithm_run(self, kh_griewank, kh_griewankc)


class KHV3TestCase(AlgorithmTestCase):
    r"""Test case for KrillHerdV3 algorithm.

    Date:
        April 2019

    Author:
        Klemen Berkovič

    See Also:
        * :class:`niapy.tests.test_algorithm.AlgorithmTestCase`
    """

    def setUp(self):
        AlgorithmTestCase.setUp(self)
        self.algo = KrillHerdV3

    def test_custom(self):
        kh_custom = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
        kh_customc = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
        AlgorithmTestCase.test_algorithm_run(self, kh_custom, kh_customc, MyBenchmark())

    def test_griewank(self):
        kh_griewank = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
        kh_griewankc = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
        AlgorithmTestCase.test_algorithm_run(self, kh_griewank, kh_griewankc)


class KHV4TestCase(AlgorithmTestCase):
    r"""Test case for KrillHerdV4 algorithm.

    Date:
        April 2019

    Author:
        Klemen Berkovič

    See Also:
        * :class:`niapy.tests.test_algorithm.AlgorithmTestCase`
    """

    def setUp(self):
        AlgorithmTestCase.setUp(self)
        self.algo = KrillHerdV4

    def test_custom(self):
        kh_custom = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
        kh_customc = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
        AlgorithmTestCase.test_algorithm_run(self, kh_custom, kh_customc, MyBenchmark())

    def test_griewank(self):
        kh_griewank = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
        kh_griewankc = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
        AlgorithmTestCase.test_algorithm_run(self, kh_griewank, kh_griewankc)


class KHV11TestCase(AlgorithmTestCase):
    r"""Test case for KrillHerdV11 algorithm.

    Date:
        April 2019

    Author:
        Klemen Berkovič

    See Also:
        * :class:`niapy.tests.test_algorithm.AlgorithmTestCase`
    """

    def setUp(self):
        AlgorithmTestCase.setUp(self)
        self.algo = KrillHerdV11

    def test_custom(self):
        kh_custom = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
        kh_customc = self.algo(population_size=10, C_a=2, C_r=0.5, seed=self.seed)
        AlgorithmTestCase.test_algorithm_run(self, kh_custom, kh_customc, MyBenchmark())

    def test_griewank(self):
        kh_griewank = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
        kh_griewankc = self.algo(population_size=10, C_a=5, C_r=0.5, seed=self.seed)
        AlgorithmTestCase.test_algorithm_run(self, kh_griewank, kh_griewankc)

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8

from math import pow
from unittest import TestCase

import numpy as np

from niapy.util.factory import get_benchmark


class TestBenchmarkFunctions(TestCase):
    """Testing the benchmarks."""

    def setUp(self):
        """Set up the tests."""

        self.D = 5
        self.array = np.asarray([0, 0, 0, 0, 0])
        self.array2 = np.asarray([1, 1, 1, 1, 1])
        self.array3 = np.asarray([420.968746, 420.968746, 420.968746, 420.968746, 420.968746])
        self.array4 = np.asarray([-2.903534, -2.903534])
        self.array5 = np.asarray([-0.5, -0.5, -0.5, -0.5, -0.5])
        self.array6 = np.asarray([-1, -1, -1, -1, -1])
        self.array7 = np.asarray([2, 2, 2, 2, 2])
        self.array8 = np.asarray(
            [7.9170526982459462172, 7.9170526982459462172, 7.9170526982459462172, 7.9170526982459462172,
             7.9170526982459462172])
        self.array9 = np.asarray([-5.12, -5.12, -5.12, -5.12, -5.12])
        self.array10 = np.asarray([1, 2, 3, 4, 5])

    def assertBounds(self, bench, lower, upper):
        """Checking the bounds.

        Args:
            bench [Benchmark]: Benchmark to test.
            lower [float]: Lower bound.
            upper [type]: Upper bound.

        Returns:
            Callable: Returns benchmarks evaluation function.
        """

        b = get_benchmark(bench)
        self.assertEqual(b.lower, lower)
        self.assertEqual(b.upper, upper)
        return b.function()

    def test_rastrigin(self):
        """Test the rastrigin benchmark."""

        rastrigin = get_benchmark('rastrigin')
        fun = rastrigin.function()
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array), 0.0)

    def test_rosenbrock(self):
        """Test the rosenbrock benchmark."""

        rosenbrock = get_benchmark('rosenbrock')
        fun = rosenbrock.function()
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array2), 0.0)

    def test_griewank(self):
        """Test the griewank benchmark."""

        griewank = get_benchmark('griewank')
        fun = griewank.function()
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array), 0.0)

    def test_sphere(self):
        """Test the sphere benchmark."""

        sphere = get_benchmark('sphere')
        fun = sphere.function()
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array), 0.0)

    def test_ackley(self):
        """Test the ackley benchmark."""

        ackley = get_benchmark('ackley')
        fun = ackley.function()
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array), 0.0, places=10)

    def test_schwefel(self):
        """Test the schwefel benchmark."""

        schwefel = get_benchmark('schwefel')
        fun = schwefel.function()
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array3), 0.0, places=3)

    def test_schwefel221(self):
        """Test the schwefel 221 benchmark."""

        schwefel221 = get_benchmark('schwefel221')
        fun = schwefel221.function()
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array), 0.0)

    def test_schwefel222(self):
        """Test the schwefel 222 benchmark."""

        schwefel222 = get_benchmark('schwefel222')
        fun = schwefel222.function()
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array), 0.0)

    def test_whitley(self):
        """Test the whitley benchmark."""

        whitley = get_benchmark('whitley')
        fun = whitley.function()
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array2), 0.0)

    def test_styblinskiTang(self):
        """Test the styblinski tang benchmark."""

        styblinskiTang = get_benchmark('styblinski_tang')
        fun = styblinskiTang.function()
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(2, self.array4), -78.332, places=3)

    def test_sumSquares(self):
        """Test the sum squares benchmark."""

        sumSquares = get_benchmark('sum_squares')
        fun = sumSquares.function()
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array), 0.0)

    def test_stepint(self):
        """Test the stepint benchmark."""

        stepint = get_benchmark('stepint')
        fun = stepint.function()
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array9), 25.0 - 6 * self.D)

    def test_step(self):
        """Test the step benchmark."""

        step = get_benchmark('step')
        fun = step.function()
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array), 0.0)

    def test_step2(self):
        """Test the step 2 benchmark."""

        step2 = get_benchmark('step2')
        fun = step2.function()
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array5), 0.0)

    def test_step3(self):
        """Test the step3 benchmark."""

        step3 = get_benchmark('step3')
        fun = step3.function()
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array), 0.0)

    def test_schumerSteiglitz(self):
        """Test the schumer steiglitz benchmark."""

        fun = self.assertBounds('schumer_steiglitz', -100, 100)
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array), 0.0)

    def test_salomon(self):
        """Test the salomon benchmark."""

        fun = self.assertBounds('salomon', -100.0, 100.0)
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array), 0.0)

    def test_quintic(self):
        """Test the quintic benchmark."""

        fun = self.assertBounds('quintic', -10.0, 10.0)
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array6), 0.0)

    def test_quintic2(self):
        """Test the quintic benchmark."""

        fun = self.assertBounds('quintic', -10.0, 10.0)
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array7), 0.0)

    def test_pinter(self):
        """Test the pinter benchmark."""

        fun = self.assertBounds('pinter', -10.0, 10.0)
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array), 0.0)

    def test_alpine1(self):
        """Test the alpine 1 benchmark."""

        fun = self.assertBounds('alpine1', -10.0, 10.0)
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array), 0.0)

    def test_alpine2(self):
        """Test the apline 2 benchmark."""

        fun = self.assertBounds('alpine2', 0.0, 10.0)
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array8), pow(2.8081311800070053291, self.D))

    def test_chungReynolds(self):
        """Test the chung reynolds benchmark."""

        fun = self.assertBounds('chung_reynolds', -100, 100)
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array), 0.0)

    def test_csendes(self):
        """Test the csendes benchmark."""

        fun = self.assertBounds('csendes', -1.0, 1.0)
        self.assertTrue(callable(fun))
        self.assertEqual(fun(self.D, self.array), 0.0)

    def test_bentcigar(self):
        """Test the bent cigar benchmark."""

        fun = self.assertBounds('bent_cigar', -100, 100)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(2, np.zeros(2)), 0.0, delta=1e-4)
        self.assertAlmostEqual(fun(10, np.zeros(10)), 0.0, delta=1e-4)
        self.assertAlmostEqual(fun(100, np.zeros(100)), 0.0, delta=1e-4)

    def test_discus(self):
        """Test the discus benchmark."""

        fun = self.assertBounds('discus', -100, 100)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array10), 1000054.0, delta=1e-4)

    def test_elliptic(self):
        """Test the elliptic benchmark."""

        fun = self.assertBounds('elliptic', -100, 100)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array10), 5129555.351959938, delta=2e6)

    def test_expanded_griewank_plus_rosenbrock(self):
        """Test the expanded griewank plus rosenbrock benchmark."""

        fun = self.assertBounds('expanded_griewank_plus_rosenbrock', -100, 100)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array), 2.2997, delta=1e2)

    def test_expanded_schaffer(self):
        """Test the expanded schaffer benchmark."""

        fun = self.assertBounds('expanded_schaffer', -100, 100)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array10), 2.616740208857464, delta=1e-4)

    def test_schaffern2(self):
        """Test the schaffer n. 2 benchmark."""
        fun = self.assertBounds('schaffer2', -100, 100)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array10), 0.02467, delta=1e-4)

    def test_schaffern4(self):
        """Test the schaffer n. 4 benchmark."""

        fun = self.assertBounds('schaffer4', -100, 100)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array10), 0.97545, delta=1e-4)

    def test_hgbat(self):
        """Test the hgbat benchmark."""

        fun = self.assertBounds('hgbat', -100, 100)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array10), 61.91502622129181, delta=60)

    def test_katsuura(self):
        """Test the katsuura benchmark."""

        fun = self.assertBounds('katsuura', -100, 100)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array10), 3837.4739882594373, delta=4000)

    def test_modifiedscwefel(self):
        """Test the modified scwefel benchmark."""

        fun = self.assertBounds('modified_schwefel', -100, 100)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array10), 6.9448853328785844, delta=350)

    def test_weierstrass(self):
        """Test the weierstrass benchmark."""

        fun = self.assertBounds('weierstrass', -100, 100)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array10), 0.0, delta=1e-4)

    def test_happyCat(self):
        """Test the happy cat benchmark."""

        fun = self.assertBounds('happy_cat', -100, 100)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array10), 15.1821333, delta=1e-4)

    def test_qing(self):
        """Test the quing benchmark."""

        fun = self.assertBounds('qing', -500, 500)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array10), 669.0, delta=1e-4)

    def test_ridge(self):
        """Test the ridge benchmark."""

        fun = self.assertBounds('ridge', -64, 64)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(self.D, self.array10), 371.0, delta=1e-4)

    def test_michalewicz(self):
        """Test the michalewicz benchmark."""

        fun = self.assertBounds('michalewicz', 0, np.pi)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(2, np.asarray([2.20, 1.57])), -1.8013, delta=1e-3)

    def test_levy(self):
        """Test the levy benchmark."""

        fun = self.assertBounds('levy', 0, np.pi)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(2, np.ones(2)), 0.0)
        self.assertAlmostEqual(fun(10, np.ones(10)), 0.0)
        self.assertAlmostEqual(fun(100, np.ones(100)), 0.0)

    def test_sphere2(self):
        """Test the sphere 2 benchmark."""

        fun = self.assertBounds('sphere2', -1, 1)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(2, np.zeros(2)), 0.0)
        self.assertAlmostEqual(fun(10, np.zeros(10)), 0.0)
        self.assertAlmostEqual(fun(100, np.zeros(100)), 0.0)

    def test_sphere3(self):
        """Test the sphere 3 benchmark."""

        fun = self.assertBounds('sphere3', -65.536, 65.536)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(2, np.zeros(2)), 0.0)
        self.assertAlmostEqual(fun(10, np.zeros(10)), 0.0)
        self.assertAlmostEqual(fun(100, np.zeros(100)), 0.0)

    def __trid_opt(self, d):
        """Trid benchmark optimum."""

        return -d * (d + 4) * (d - 1) / 6

    def __trid_opt_sol(self, d):
        """Trid optimal solution."""

        return np.asarray([i * (d + 1 - i) for i in range(1, d + 1)])

    def test_trid(self):
        """Test the trid benchmark."""

        fun = self.assertBounds('trid', -2 ** 2, 2 ** 2)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(2, self.__trid_opt_sol(2)), self.__trid_opt(2))
        self.assertAlmostEqual(fun(10, self.__trid_opt_sol(10)), self.__trid_opt(10))
        self.assertAlmostEqual(fun(100, self.__trid_opt_sol(100)), self.__trid_opt(100))

    def __perm_opt_sol(self, d):
        """The perm optimal solution."""

        return np.asarray([1 / i for i in range(1, d + 1)])

    def test_perm(self):
        """Test the perm bencmark."""

        fun = self.assertBounds('perm', -10, 10)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(2, self.__perm_opt_sol(2)), .0)
        self.assertAlmostEqual(fun(10, self.__perm_opt_sol(10)), .0)
        self.assertAlmostEqual(fun(100, self.__perm_opt_sol(100)), .0)

    def test_zakharov(self):
        """Test the zakharov benchmark."""

        fun = self.assertBounds('zakharov', -5, 10)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(2, np.zeros(2)), .0)
        self.assertAlmostEqual(fun(10, np.zeros(10)), .0)
        self.assertAlmostEqual(fun(100, np.zeros(100)), .0)

    def __dixonprice_opt_sol(self, d):
        """The dixon price optimal solution."""
        return np.asarray([2 ** (-(2 ** i - 2) / 2 ** i) for i in range(1, d + 1)])

    def test_dixonprice(self):
        """Test the dixon price benchmark."""

        fun = self.assertBounds('dixon_price', -10, 10)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(2, self.__dixonprice_opt_sol(2)), .0)
        self.assertAlmostEqual(fun(10, self.__dixonprice_opt_sol(10)), .0)
        self.assertAlmostEqual(fun(100, self.__dixonprice_opt_sol(100)), .0)

    def test_powell(self):
        """Tests the powell benchmark."""

        fun = self.assertBounds('powell', -4, 5)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(2, np.zeros(2)), .0)
        self.assertAlmostEqual(fun(10, np.zeros(10)), .0)
        self.assertAlmostEqual(fun(100, np.zeros(100)), .0)

    def test_cosinemixture(self):
        """Test the cosine mixture benchmark."""

        fun = self.assertBounds('cosine_mixture', -1, 1)
        self.assertTrue(callable(fun))
        self.assertAlmostEqual(fun(2, np.zeros(2)), -.1 * 2)
        self.assertAlmostEqual(fun(10, np.zeros(10)), -.1 * 10)
        self.assertAlmostEqual(fun(100, np.zeros(100)), -.1 * 100)

    def test_infinity(self):
        """Test the infinity benchmark."""

        fun = self.assertBounds('infinity', -1, 1)
        self.assertTrue(callable(fun))
        sizes = [2, 10, 100]
        defaults = np.seterr('raise')
        for size in sizes:
            with self.assertRaises(FloatingPointError):
                fun(size, np.zeros(size))
            self.assertAlmostEqual(fun(size, np.ones(size)), 2.84147098480789650665250232163 * size)
        np.seterr(**defaults)



# encoding=utf8

"""Implementations of Cosine mixture functions."""

import numpy as np
from niapy.benchmarks.benchmark import Benchmark

__all__ = ['CosineMixture']


class CosineMixture(Benchmark):
    r"""Implementations of Cosine mixture function.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Cosine Mixture Function**

        :math:`f(\textbf{x}) = - 0.1 \sum_{i = 1}^D \cos (5 \pi x_i) - \sum_{i = 1}^D x_i^2`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-1, 1]`, for all :math:`i = 1, 2,..., D`.

        **Global maximum:**
        :math:`f(x^*) = -0.1 D`, at :math:`x^* = (0.0,...,0.0)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = - 0.1 \sum_{i = 1}^D \cos (5 \pi x_i) - \sum_{i = 1}^D x_i^2$

        Equation:
            \begin{equation} f(\textbf{x}) = - 0.1 \sum_{i = 1}^D \cos (5 \pi x_i) - \sum_{i = 1}^D x_i^2 \end{equation}

        Domain:
            $-1 \leq x_i \leq 1$

    Reference:
        http://infinity77.net/global_optimization/test_functions_nd_C.html#go_benchmark.CosineMixture

    """

    Name = ['CosineMixture']

    def __init__(self, lower=-1.0, upper=1.0):
        r"""Initialize of Cosine mixture benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = - 0.1 \sum_{i = 1}^D \cos (5 \pi x_i) - \sum_{i = 1}^D x_i^2$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def f(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            v1, v2 = 0.0, 0.0
            for i in range(dimension):
                v1, v2 = v1 + np.cos(5 * np.pi * x[i]), v2 + x[i] ** 2
            return -0.1 * v1 - v2

        return f

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8
from unittest import TestCase

import numpy as np
from numpy.random import default_rng

from niapy.benchmarks import Benchmark
from niapy.task import StoppingTask, ThrowingTask
from niapy.util import full_array, FesException, GenException, RefException


class MyBenchmark(Benchmark):
    def __init__(self):
        super().__init__(-10, 10)

    def function(self):
        def evaluate(D, x):
            return sum(x ** 2)

        return evaluate


class StoppingTaskBaseTestCase(TestCase):
    r"""Test case for testing `Task`, `StoppingTask` and `CountingTask` classes.

    Date:
        April 2019

    Author:
        Klemen Berkovič

    See Also:
        * :class:`niapy.util.Task`
        * :class:`niapy.util.CountingTask`
        * :class:`niapy.util.StoppingTask`
    """

    def setUp(self):
        self.D = 6
        self.Lower, self.Upper = [2, 1, 1], [10, 10, 2]
        self.task = StoppingTask(dimension=self.D, lower=self.Lower, upper=self.Upper)

    def test_dim_ok(self):
        self.assertEqual(self.D, self.task.dimension)
        self.assertEqual(self.D, self.task.dimension)

    def test_lower(self):
        self.assertTrue(np.array_equal(full_array(self.Lower, self.D), self.task.lower))
        self.assertTrue(np.array_equal(full_array(self.Lower, self.D), self.task.lower))

    def test_upper(self):
        self.assertTrue(np.array_equal(full_array(self.Upper, self.D), self.task.upper))
        self.assertTrue(np.array_equal(full_array(self.Upper, self.D), self.task.upper))

    def test_range(self):
        self.assertTrue(
            np.array_equal(full_array(self.Upper, self.D) - full_array(self.Lower, self.D), self.task.range))
        self.assertTrue(
            np.array_equal(full_array(self.Upper, self.D) - full_array(self.Lower, self.D), self.task.range))

    def test_ngens(self):
        self.assertEqual(np.inf, self.task.max_iters)

    def test_nfess(self):
        self.assertEqual(np.inf, self.task.max_evals)

    def test_stop_cond(self):
        self.assertFalse(self.task.stopping_condition())

    def test_stop_condi(self):
        self.assertFalse(self.task.stopping_condition_iter())

    def test_eval(self):
        self.assertRaises(AttributeError, lambda: self.task.eval([]))

    def test_evals(self):
        self.assertEqual(0, self.task.evals)

    def test_iters(self):
        self.assertEqual(0, self.task.iters)

    def test_next_iter(self):
        self.assertEqual(None, self.task.next_iter())

    def test_is_feasible(self):
        self.assertFalse(self.task.is_feasible(full_array([1, 2, 3], self.D)))


class StoppingTaskTestCase(TestCase):
    r"""Test case for testing `Task`, `StoppingTask` and `CountingTask` classes.

    Date:
        April 2019

    Author:
        Klemen Berkovič

    See Also:
        * :class:`niapy.util.Task`
        * :class:`niapy.util.CountingTask`
        * :class:`niapy.util.StoppingTask`
    """

    def setUp(self):
        self.D, self.nFES, self.nGEN = 10, 10, 10
        self.t = StoppingTask(max_evals=self.nFES, max_iters=self.nGEN, cutoff_value=1, dimension=self.D,
                              benchmark=MyBenchmark())

    def test_isFeasible(self):
        x = np.full(self.D, 10)
        self.assertTrue(self.t.is_feasible(x))
        x = np.full(self.D, -10)
        self.assertTrue(self.t.is_feasible(x))
        x = default_rng().uniform(-10, 10, self.D)
        self.assertTrue(self.t.is_feasible(x))
        x = np.full(self.D, -20)
        self.assertFalse(self.t.is_feasible(x))
        x = np.full(self.D, 20)
        self.assertFalse(self.t.is_feasible(x))

    def test_nextIter(self):
        for i in range(self.nGEN):
            self.assertFalse(self.t.stopping_condition())
            self.t.next_iter()
        self.assertTrue(self.t.stopping_condition())

    def test_stopCondI(self):
        for i in range(self.nGEN):
            self.assertFalse(self.t.stopping_condition_iter(), msg='Error at %s iteration!!!' % i)
        self.assertTrue(self.t.stopping_condition_iter())

    def test_eval(self):
        x = np.ones(self.D)
        for i in range(self.nFES):
            self.assertAlmostEqual(self.t.eval(x), self.D, msg='Error at %s iteration!!!' % i)
        self.assertTrue(self.t.stopping_condition())

    def test_eval_over_nFES(self):
        x = np.ones(self.D)
        for i in range(self.nFES):
            self.t.eval(x)
        self.assertEqual(np.inf, self.t.eval(x))
        self.assertTrue(self.t.stopping_condition())

    def test_eval_over_nGEN(self):
        x = np.ones(self.D)
        for i in range(self.nGEN):
            self.t.next_iter()
        self.assertEqual(np.inf, self.t.eval(x))
        self.assertTrue(self.t.stopping_condition())

    def test_nFES_count(self):
        x = np.ones(self.D)
        for i in range(self.nFES):
            self.t.eval(x)
            self.assertEqual(self.t.evals, i + 1, 'Error at %s. evaluation' % (i + 1))

    def test_nGEN_count(self):
        x = np.ones(self.D)
        for i in range(self.nGEN):
            self.t.next_iter()
            self.assertEqual(self.t.iters, i + 1, 'Error at %s. iteration' % (i + 1))

    def test_stopCond_evals(self):
        x = np.ones(self.D)
        for i in range(self.nFES - 1):
            self.t.eval(x)
            self.assertFalse(self.t.stopping_condition())
        self.t.eval(x)
        self.assertTrue(self.t.stopping_condition())

    def test_stopCond_iters(self):
        x = np.ones(self.D)
        for i in range(self.nGEN - 1):
            self.t.next_iter()
            self.assertFalse(self.t.stopping_condition())
        self.t.next_iter()
        self.assertTrue(self.t.stopping_condition())

    def test_stopCond_refValue(self):
        x = np.ones(self.D)
        for i in range(self.nGEN - 5):
            self.assertFalse(self.t.stopping_condition())
            self.assertEqual(self.D, self.t.eval(x))
            self.t.next_iter()
        x = np.zeros(self.D)
        self.assertEqual(0, self.t.eval(x))
        self.assertTrue(self.t.stopping_condition())
        self.assertEqual(self.nGEN - 5, self.t.iters)

    def test_print_conv_one(self):
        r1, r2 = [], []
        for i in range(self.nFES):
            x = np.full(self.D, 10 - i)
            r1.append(i + 1), r2.append(self.t.eval(x))
        t_r1, t_r2 = self.t.return_conv()
        self.assertTrue(np.array_equal(r1, t_r1))
        self.assertTrue(np.array_equal(r2, t_r2))

    def test_print_conv_two(self):
        r1, r2 = [], []
        for i in range(self.nFES):
            x = np.full(self.D, 10 - i if i not in (3, 4, 5) else 4)
            r1.append(i + 1), r2.append(self.t.eval(x))
        t_r1, t_r2 = self.t.return_conv()
        self.assertTrue(np.array_equal(r2, t_r2))
        self.assertTrue(np.array_equal(r1, t_r1))


class ThrowingTaskTestCase(TestCase):
    r"""Test case for testing `ThrowingTask` class.

    Date:
        April 2019

    Author:
        Klemen Berkovič

    See Also:
        * :class:`niapy.util.ThrowingTask`
    """

    def setUp(self):
        self.D, self.nFES, self.nGEN = 10, 10, 10
        self.t = ThrowingTask(dimension=self.D, max_evals=self.nFES, max_iters=self.nGEN, cutoff_value=0,
                              benchmark=MyBenchmark())

    def test_isFeasible(self):
        x = np.full(self.D, 10)
        self.assertTrue(self.t.is_feasible(x))
        x = np.full(self.D, -10)
        self.assertTrue(self.t.is_feasible(x))
        x = default_rng().uniform(-10, 10, self.D)
        self.assertTrue(self.t.is_feasible(x))
        x = np.full(self.D, -20)
        self.assertFalse(self.t.is_feasible(x))
        x = np.full(self.D, 20)
        self.assertFalse(self.t.is_feasible(x))

    def test_nextIter(self):
        for i in range(self.nGEN):
            self.assertFalse(self.t.stopping_condition())
            self.t.next_iter()
        self.assertTrue(self.t.stopping_condition())

    def test_stopCondI(self):
        for i in range(self.nGEN):
            self.assertFalse(self.t.stopping_condition_iter())
        self.assertTrue(self.t.stopping_condition_iter())

    def test_eval(self):
        x = np.ones(self.D)
        for i in range(self.nFES):
            self.assertAlmostEqual(self.t.eval(x), self.D, msg='Error at %s iteration!!!' % i)
        self.assertRaises(FesException, lambda: self.t.eval(x))

    def test_eval_over_nFES(self):
        x = np.ones(self.D)
        for i in range(self.nFES):
            self.t.eval(x)
        self.assertRaises(FesException, lambda: self.t.eval(x))

    def test_eval_over_nGEN(self):
        x = np.ones(self.D)
        for i in range(self.nGEN):
            self.t.next_iter()
        self.assertRaises(GenException, lambda: self.t.eval(x))

    def test_nFES_count(self):
        x = np.ones(self.D)
        for i in range(self.nFES):
            self.t.eval(x)
            self.assertEqual(self.t.evals, i + 1, 'Error at %s. evaluation' % (i + 1))

    def test_nGEN_count(self):
        x = np.ones(self.D)
        for i in range(self.nGEN):
            self.t.next_iter()
            self.assertEqual(self.t.iters, i + 1, 'Error at %s. iteration' % (i + 1))

    def test_stopCond_evals(self):
        x = np.ones(self.D)
        for i in range(self.nFES - 1):
            self.t.eval(x)
            self.assertFalse(self.t.stopping_condition())
        self.t.eval(x)
        self.assertTrue(self.t.stopping_condition())

    def test_stopCond_iters(self):
        x = np.ones(self.D)
        for i in range(self.nGEN - 1):
            self.t.next_iter()
            self.assertFalse(self.t.stopping_condition())
        self.t.next_iter()
        self.assertTrue(self.t.stopping_condition())

    def test_stopCond_refValue(self):
        x = np.ones(self.D)
        for i in range(self.nGEN - 5):
            self.assertFalse(self.t.stopping_condition())
            self.assertEqual(self.D, self.t.eval(x))
            self.t.next_iter()
        x = np.zeros(self.D)
        self.assertEqual(0, self.t.eval(x))
        self.assertRaises(RefException, lambda: self.t.eval(x))

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8

"""Implementations of Powell function."""

from niapy.benchmarks.benchmark import Benchmark

__all__ = ['Powell']


class Powell(Benchmark):
    r"""Implementations of Powell functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Powell Function**

        :math:`f(\textbf{x}) = \sum_{i = 1}^{D / 4} \left( (x_{4 i - 3} + 10 x_{4 i - 2})^2 + 5 (x_{4 i - 1} - x_{4 i})^2 + (x_{4 i - 2} - 2 x_{4 i - 1})^4 + 10 (x_{4 i - 3} - x_{4 i})^4 \right)`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-4, 5]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        :math:`f(\textbf{x}^*) = 0` at :math:`\textbf{x}^* = (0, \cdots, 0)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = \sum_{i = 1}^{D / 4} \left( (x_{4 i - 3} + 10 x_{4 i - 2})^2 + 5 (x_{4 i - 1} - x_{4 i})^2 + (x_{4 i - 2} - 2 x_{4 i - 1})^4 + 10 (x_{4 i - 3} - x_{4 i})^4 \right)$

        Equation:
            \begin{equation} f(\textbf{x}) = \sum_{i = 1}^{D / 4} \left( (x_{4 i - 3} + 10 x_{4 i - 2})^2 + 5 (x_{4 i - 1} - x_{4 i})^2 + (x_{4 i - 2} - 2 x_{4 i - 1})^4 + 10 (x_{4 i - 3} - x_{4 i})^4 \right) \end{equation}

        Domain:
            $-4 \leq x_i \leq 5$

    Reference:
        https://www.sfu.ca/~ssurjano/powell.html

    """

    Name = ['Powell']

    def __init__(self, lower=-4.0, upper=5.0):
        r"""Initialize of Powell benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = \sum_{i = 1}^{D / 4} \left( (x_{4 i - 3} + 10 x_{4 i - 2})^2 + 5 (x_{4 i - 1} - x_{4 i})^2 + (x_{4 i - 2} - 2 x_{4 i - 1})^4 + 10 (x_{4 i - 3} - x_{4 i})^4 \right)$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def f(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            v = 0.0
            for i in range(1, (dimension // 4) + 1):
                v += (x[4 * i - 4] + 10 * x[4 * i - 3]) ** 2 + 5 * (x[4 * i - 2] - x[4 * i - 1]) ** 2 + (x[4 * i - 3] - 2 * x[4 * i - 2]) ** 4 + 10 * (x[4 * i - 4] - x[4 * i - 1]) ** 4
            return v

        return f

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3



# encoding=utf8

"""Implementation of Ackley benchmark."""

import numpy as np

from niapy.benchmarks.benchmark import Benchmark

__all__ = ['Ackley']


class Ackley(Benchmark):
    r"""Implementation of Ackley function.

    Date: 2018

    Author: Lucija Brezočnik

    License: MIT

    Function: **Ackley function**

        :math:`f(\mathbf{x}) = -a\;\exp\left(-b \sqrt{\frac{1}{D}\sum_{i=1}^D x_i^2}\right)
        - \exp\left(\frac{1}{D}\sum_{i=1}^D \cos(c\;x_i)\right) + a + \exp(1)`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-32.768, 32.768]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(\textbf{x}^*) = 0`, at  :math:`x^* = (0,...,0)`

    LaTeX formats:
        Inline:
            $f(\mathbf{x}) = -a\;\exp\left(-b \sqrt{\frac{1}{D}
            \sum_{i=1}^D x_i^2}\right) - \exp\left(\frac{1}{D}
            \sum_{i=1}^D cos(c\;x_i)\right) + a + \exp(1)$

        Equation:
            \begin{equation}f(\mathbf{x}) =
            -a\;\exp\left(-b \sqrt{\frac{1}{D} \sum_{i=1}^D x_i^2}\right) -
            \exp\left(\frac{1}{D} \sum_{i=1}^D \cos(c\;x_i)\right) +
            a + \exp(1) \end{equation}

        Domain:
            $-32.768 \leq x_i \leq 32.768$

    Reference:
        https://www.sfu.ca/~ssurjano/ackley.html

    """

    Name = ['Ackley']

    def __init__(self, lower=-32.768, upper=32.768):
        r"""Initialize of Ackley benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\mathbf{x}) = -a\;\exp\left(-b \sqrt{\frac{1}{D}
                \sum_{i=1}^D x_i^2}\right) - \exp\left(\frac{1}{D}
                \sum_{i=1}^D \cos(c\;x_i)\right) + a + \exp(1)$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def evaluate(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            a = 20  # Recommended variable value
            b = 0.2  # Recommended variable value
            c = 2 * np.pi  # Recommended variable value

            val1 = 0.0
            val2 = 0.0

            for i in range(dimension):
                val1 += x[i] ** 2
                val2 += np.cos(c * x[i])

            temp1 = -b * np.sqrt(val1 / dimension)
            temp2 = val2 / dimension

            val = -a * np.exp(temp1) - np.exp(temp2) + a + np.exp(1)

            return val

        return evaluate



# encoding=utf8

"""Implementations of Levy function."""

import numpy as np
from niapy.benchmarks.benchmark import Benchmark

__all__ = ['Levy']


class Levy(Benchmark):
    r"""Implementations of Levy functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Levy Function**

        :math:`f(\textbf{x}) = \sin^2 (\pi w_1) + \sum_{i = 1}^{D - 1} (w_i - 1)^2 \left( 1 + 10 \sin^2 (\pi w_i + 1) \right) + (w_d - 1)^2 (1 + \sin^2 (2 \pi w_d)) \\ w_i = 1 + \frac{x_i - 1}{4}`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-10, 10]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        :math:`f(\textbf{x}^*) = 0` at :math:`\textbf{x}^* = (1, \cdots, 1)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = \sin^2 (\pi w_1) + \sum_{i = 1}^{D - 1} (w_i - 1)^2 \left( 1 + 10 \sin^2 (\pi w_i + 1) \right) + (w_d - 1)^2 (1 + \sin^2 (2 \pi w_d)) \\ w_i = 1 + \frac{x_i - 1}{4}$

        Equation:
            \begin{equation} f(\textbf{x}) = \sin^2 (\pi w_1) + \sum_{i = 1}^{D - 1} (w_i - 1)^2 \left( 1 + 10 \sin^2 (\pi w_i + 1) \right) + (w_d - 1)^2 (1 + \sin^2 (2 \pi w_d)) \\ w_i = 1 + \frac{x_i - 1}{4} \end{equation}

        Domain:
            $-10 \leq x_i \leq 10$

    Reference:
        https://www.sfu.ca/~ssurjano/levy.html

    """

    Name = ['Levy']

    def __init__(self, lower=0.0, upper=np.pi):
        r"""Initialize of Levy benchmark.

        Args:
            lower (Optional[float]): Lower bound of problem.
            upper (Optional[float]): Upper bound of problem.

        See Also:
            :func:`niapy.benchmarks.Benchmark.__init__`

        """
        super().__init__(lower, upper)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = \sin^2 (\pi w_1) + \sum_{i = 1}^{D - 1} (w_i - 1)^2 \left( 1 + 10 \sin^2 (\pi w_i + 1) \right) + (w_d - 1)^2 (1 + \sin^2 (2 \pi w_d)) \\ w_i = 1 + \frac{x_i - 1}{4}$'''

    def function(self):
        r"""Return benchmark evaluation function.

        Returns:
            Callable[[int, Union[int, float, List[int, float], numpy.ndarray]], float]: Fitness function.

        """
        def w(x):
            return 1 + (x - 1) / 4

        def f(dimension, x):
            r"""Fitness function.

            Args:
                dimension (int): Dimensionality of the problem
                x (Union[int, float, List[int, float], numpy.ndarray]): Solution to check.

            Returns:
                float: Fitness value for the solution.

            """
            v = 0.0
            for i in range(dimension - 1):
                v += (w(x[i]) - 1) ** 2 * (1 + 10 * np.sin(np.pi * w(x[i]) + 1) ** 2) + (w(x[-1]) - 1) ** 2 * (1 + np.sin(2 * np.pi * w(x[-1]) ** 2))
            return np.sin(np.pi * w(x[0])) ** 2 + v

        return f

# vim: tabstop=3 noexpandtab shiftwidth=3 softtabstop=3
